import { OnomyConstants, OnexConstants, OnomyProposals, OnexProposals } from '@onomy/client';
export { CosmosClient, OnexClient, OnomyClient } from '@onomy/client';
import { useAsyncStatuses, useAsyncPoll, useSubscribedValue, useAsyncProcess } from '@onomy/react-utils';
export { useAsyncPoll, useAsyncProcess, useAsyncStatuses, useAsyncValue } from '@onomy/react-utils';
import { useDetailedBalance, useSupportedAssetList, useOnomyHub, serializeAsset, useAddress, OnomyHubProvider, useHoldings, useDelegations, useDetailedBalances, useFiatTotal, useAssetInBalances, useTransferAssetForm, useClaimRewardForm, useDelegateAssetForm, useMoveDelegationForm, useDelegation, useProposals, useAssetBalance, useSubmitProposalForm, useProposal, useStakingAssets, useProposalMinDeposit, useFixedAmountInput, useGasForm, useGovVoteForm, useAsset as useAsset$1, useBalanceTotals, useBridgeForm } from '@onomy/react-hub';
export { AmountDisplay, BigNumDisplay, FiatValueDisplay, RatioAsPercentage, TruncatedAddress, serializeAddress, serializeAsset, serializeBalance, serializeBalances, useAddress, useConnectedWallet, useWalletsForChain } from '@onomy/react-hub';
import { OnomyHubConstants, OnomyHubUtils, AbstractThrottledAsyncCache, OnomyHub } from '@onomy/hub';
import BigNumber from 'bignumber.js';
import React, { useCallback, useMemo, useEffect, createContext, useContext, useState } from 'react';
import { gql, ApolloClient, InMemoryCache } from '@apollo/client';
import { Metadata } from '@onomy/telescope/dist/codegen/cosmos/bank/v1beta1/bank';

const onomyVestingSchedules = [
  { amount: new BigNumber(29367338), duration: 36, cliff: 12 },
  { amount: new BigNumber(3595397), duration: 30, cliff: 12 },
  { amount: new BigNumber(12302874), duration: 24, cliff: 12 }
];
const DEVNET = {
  onomyChainInfo: OnomyConstants.CHAIN_INFO.TESTNET,
  onomyVestingSchedules,
  onomyStartDate: new Date("2022-11-29"),
  onomyDaoAddress: "onomy1vwr8z00ty7mqnk4dtchr9mn9j96nuh6wrlww93",
  onomyGraphqlEndpoint: "https://api.thegraph.com/subgraphs/name/onomyprotocol/ograph-devnet",
  onexChainInfo: OnexConstants.CHAIN_INFO.DEVNET,
  onexGraphqlEndpoint: "https://graphql-onex-devnet.onomy.io/subgraphs/name/onomyprotocol/mgraph",
  ethNetworkName: "goerli",
  ethWsUrl: "",
  bNomContractAddress: "0x8EFe26D6839108E831D3a37cA503eA4F136A8E73",
  ethNomContractAddress: "0x9653a2013e23ae976d34543572e3e72f8578e4c6",
  bondContractAddress: "0xD179c5BEd30cADE4E62d53DD89240745Fb4C0CC2",
  gravityContractAddress: "0x18619DE15bDd14b0360e82e2746aAf77B17b3925",
  wEthUsdUniswapPoolAddress: "",
  ethBridgeWaitBlocks: 35
};
const TESTNET = {
  onomyChainInfo: OnomyConstants.CHAIN_INFO.TESTNET,
  onomyVestingSchedules,
  onomyStartDate: new Date("2022-11-29"),
  onomyDaoAddress: "onomy1vwr8z00ty7mqnk4dtchr9mn9j96nuh6wrlww93",
  onomyGraphqlEndpoint: "https://api.thegraph.com/subgraphs/name/onomyprotocol/ograph-testnet",
  onexChainInfo: OnexConstants.CHAIN_INFO.TESTNET,
  onexGraphqlEndpoint: "https://graphql-onex-testnet.onomy.io/subgraphs/name/onomyprotocol/mgraph",
  ethNetworkName: "goerli",
  ethWsUrl: "",
  bNomContractAddress: "0x8EFe26D6839108E831D3a37cA503eA4F136A8E73",
  ethNomContractAddress: "",
  bondContractAddress: "0xD179c5BEd30cADE4E62d53DD89240745Fb4C0CC2",
  gravityContractAddress: "0xBA7cDCD837757EB67Ea889d168Aba1a1Fa7C1340",
  wEthUsdUniswapPoolAddress: "",
  ethBridgeWaitBlocks: 64
};
const MAINNET = {
  onomyChainInfo: OnomyConstants.CHAIN_INFO.MAINNET,
  onomyVestingSchedules,
  onomyStartDate: new Date("2022-11-29"),
  onomyDaoAddress: "onomy1vwr8z00ty7mqnk4dtchr9mn9j96nuh6wrlww93",
  onomyGraphqlEndpoint: "https://api.thegraph.com/subgraphs/name/onomyprotocol/ograph-mainnet",
  onexChainInfo: void 0,
  onexGraphqlEndpoint: "",
  ethNetworkName: "homestead",
  ethWsUrl: "",
  bNomContractAddress: "0x4fdF157C6860F33232608198419c74ED446Ca577",
  ethNomContractAddress: "",
  bondContractAddress: "0x264C13cfEd981e3137Fb43B198D14D8D5D64977E",
  gravityContractAddress: "0x85391fd149282C50B103aC810430F887685d575C",
  wEthUsdUniswapPoolAddress: "0x4e68ccd3e89f51c3074ca5072bbac773960dfa36",
  ethBridgeWaitBlocks: 64
};
const OnomyEcosystemConfigs = {
  DEVNET,
  TESTNET,
  MAINNET
};
function setOnomyEcosystemConfig(configName, overrides) {
  const config = { ...OnomyEcosystemConfigs[configName], ...overrides != null ? overrides : {} };
  OnomyHubConstants.updateAssetData({
    bNomAsset: {
      id: config.bNomContractAddress
    },
    ethNomAsset: {
      id: config.ethNomContractAddress
    },
    ethChainId: config.ethNetworkName
  });
  return config;
}

function useOnexClient() {
  var _a;
  const { hub } = useOnomyHub();
  const onex = (_a = hub == null ? void 0 : hub.chains.onex) == null ? void 0 : _a.client;
  if (!onex)
    return null;
  return onex;
}
function useOnexBurned() {
  var _a;
  const onex = useOnexClient();
  const chain = onex == null ? void 0 : onex.getBlockchain();
  const [assets, assetsStatus] = useSupportedAssetList();
  const [burnedRaw, burnedStatus] = useAsyncPoll(useCallback(async () => {
    var _a2;
    return (_a2 = onex == null ? void 0 : onex.burned()) != null ? _a2 : null;
  }, [onex]), null);
  const asset = useMemo(() => {
    if (!burnedRaw || !(chain == null ? void 0 : chain.name))
      return OnomyHubConstants.UNKNOWN_BALANCE.asset;
    const burnAsset = assets.find((a) => a.id === burnedRaw.denom && a.chain.name === chain.name);
    return burnAsset != null ? burnAsset : OnomyHubConstants.UNKNOWN_BALANCE.asset;
  }, [assets, burnedRaw, chain == null ? void 0 : chain.name]);
  const [amount, amountStatus] = useDetailedBalance(asset, (_a = burnedRaw == null ? void 0 : burnedRaw.amount) != null ? _a : "0");
  const status = useAsyncStatuses([assetsStatus, burnedStatus, amountStatus]);
  return [amount, status];
}
function useOnexPool({ baseDenom = "anom", quoteDenom = "usdt" }) {
  const onex = useOnexClient();
  const pair = useMemo(() => [baseDenom, quoteDenom].sort().join(","), [baseDenom, quoteDenom]);
  const cb = useCallback(async () => {
    var _a, _b;
    try {
      const res = (_a = await (onex == null ? void 0 : onex.pool(pair))) != null ? _a : null;
      if (!res || res.drops === "0")
        return null;
      return res;
    } catch (e) {
      if ((_b = e == null ? void 0 : e.message) == null ? void 0 : _b.match(/not found/))
        return null;
      throw e;
    }
  }, [pair, onex]);
  return useAsyncPoll(cb, null);
}
function useOnexPairMembers({ baseDenom = "anom", quoteDenom = "usdt" }) {
  const onex = useOnexClient();
  const cb = useCallback(async () => {
    var _a, _b, _c;
    try {
      const [memberBase, memberQuote] = await Promise.all([
        (_a = onex == null ? void 0 : onex.member({
          denomA: quoteDenom,
          denomB: baseDenom
        })) != null ? _a : Promise.resolve(null),
        (_b = onex == null ? void 0 : onex.member({
          denomA: baseDenom,
          denomB: quoteDenom
        })) != null ? _b : Promise.resolve(null)
      ]);
      return { memberBase, memberQuote };
    } catch (e) {
      if ((_c = e == null ? void 0 : e.message) == null ? void 0 : _c.match(/not found/))
        return { memberBase: null, memberQuote: null };
      throw e;
    }
  }, [onex, baseDenom, quoteDenom]);
  return useAsyncPoll(cb, { memberBase: null, memberQuote: null });
}
async function includePoolDetails({
  hub,
  onex,
  baseAsset,
  quoteAsset,
  pool,
  memberBase,
  memberQuote
}) {
  if (!hub)
    throw new Error("Hub is null");
  const [memberBaseTotal, memberQuoteTotal, leaders] = await Promise.all([
    hub.balances.includeDetails({
      amount: BigInt(memberBase.balance),
      asset: baseAsset
    }),
    hub.balances.includeDetails({
      amount: BigInt(memberQuote.balance),
      asset: quoteAsset
    }),
    Promise.all(pool.leaders.map(async (leader) => {
      const amounts = await (async () => {
        var _a;
        const fallback = {
          denom1: baseAsset.id,
          denom2: quoteAsset.id,
          amount1: "0",
          amount2: "0"
        };
        try {
          return (_a = await (onex == null ? void 0 : onex.dropsToCoins(baseAsset.id, quoteAsset.id, leader.drops))) != null ? _a : fallback;
        } catch (e) {
          console.warn("Error fetching dropsToCoins", { pair: pool.pair, drops: leader.drops }, e);
          return fallback;
        }
      })();
      const leaderBaseAmountFixed = amounts.amount1;
      const leaderQuoteAmountFixed = amounts.amount2;
      const [baseAmount, quoteAmount] = await Promise.all([
        hub.balances.includeDetails({
          amount: BigInt(leaderBaseAmountFixed),
          asset: baseAsset
        }),
        hub.balances.includeDetails({
          amount: BigInt(leaderQuoteAmountFixed),
          asset: quoteAsset
        })
      ]);
      return {
        ...leader,
        baseAmount,
        quoteAmount
      };
    }))
  ]);
  return {
    ...pool,
    baseAsset,
    quoteAsset,
    fiatTotal: memberBaseTotal.fiatValue + memberQuoteTotal.fiatValue,
    fiatCurrency: memberBaseTotal.fiatCurrency,
    leaders,
    memberBase: {
      ...memberBase,
      total: memberBaseTotal
    },
    memberQuote: {
      ...memberQuote,
      total: memberQuoteTotal
    }
  };
}
function useOnexPoolDetail({
  baseAsset,
  quoteAsset
}) {
  const baseDenom = baseAsset.id;
  const quoteDenom = quoteAsset.id;
  const { hub } = useOnomyHub();
  const onex = useOnexClient();
  const [pool, poolStatus] = useOnexPool({ baseDenom, quoteDenom });
  const [{ memberBase, memberQuote }, membersStatus] = useOnexPairMembers({
    baseDenom,
    quoteDenom
  });
  const detailedPoolCb = useCallback(async () => {
    if (!pool || !memberBase || !memberQuote || !hub)
      return null;
    return includePoolDetails({ hub, onex, baseAsset, quoteAsset, pool, memberBase, memberQuote });
  }, [hub, onex, baseAsset, quoteAsset, pool, memberBase, memberQuote]);
  const [detailedPool, detailedPoolStatus] = useAsyncPoll(detailedPoolCb, null);
  const combinedStatus = useAsyncStatuses([poolStatus, membersStatus, detailedPoolStatus]);
  return [detailedPool, combinedStatus];
}
function useOnexPoolPrice({ baseDenom = "anom", quoteDenom = "usdt" }) {
  const onex = useOnexClient();
  const cb = useCallback(async () => {
    var _a, _b;
    try {
      return (_a = await (onex == null ? void 0 : onex.getPoolDisplayPrice({ baseDenom, quoteDenom }))) != null ? _a : "0";
    } catch (e) {
      if ((_b = e == null ? void 0 : e.message) == null ? void 0 : _b.match(/not found/))
        return "0";
      throw e;
    }
  }, [onex, baseDenom, quoteDenom]);
  return useAsyncPoll(cb, "0");
}
function useFixedAmountFromDisplay(displayValue, asset) {
  return useMemo(() => {
    const bn = new BigNumber(displayValue);
    if (bn.isNaN())
      return "0";
    return bn.shiftedBy(asset.meta.decimalPlaces).toFixed(0);
  }, [displayValue, asset.meta.decimalPlaces]);
}
function useOnexDetailedPoolPrice({
  baseAsset,
  quoteAsset
}) {
  const baseDenom = baseAsset.id;
  const quoteDenom = quoteAsset.id;
  const [currentPrice, currentPriceStatus] = useOnexPoolPrice({ baseDenom, quoteDenom });
  const currentPriceFixed = useFixedAmountFromDisplay(currentPrice, quoteAsset);
  const [currentPricePerBase, currentPricePerBaseStatus] = useDetailedBalance(quoteAsset, currentPriceFixed);
  const status = useAsyncStatuses([currentPriceStatus, currentPricePerBaseStatus]);
  return [currentPricePerBase, status];
}
function useOnexLastExecutionPrice({ baseDenom = "anom", quoteDenom = "usdt" }) {
  const onex = useOnexClient();
  const cb = useCallback(async () => {
    var _a, _b;
    try {
      if (!baseDenom || !quoteDenom)
        return "0";
      return (_a = await (onex == null ? void 0 : onex.getLastTradeDisplayPrice({ baseDenom, quoteDenom }))) != null ? _a : "0";
    } catch (e) {
      if ((_b = e == null ? void 0 : e.message) == null ? void 0 : _b.match(/not found/))
        return "0";
      throw e;
    }
  }, [onex, baseDenom, quoteDenom]);
  return useAsyncPoll(cb, "0");
}
function useOnexDetailedLastExecutionPrice({
  baseAsset,
  quoteAsset
}) {
  const baseDenom = baseAsset == null ? void 0 : baseAsset.id;
  const quoteDenom = quoteAsset == null ? void 0 : quoteAsset.id;
  const [currentPrice, currentPriceStatus] = useOnexLastExecutionPrice({ baseDenom, quoteDenom });
  const currentPriceFixed = useFixedAmountFromDisplay(currentPrice, quoteAsset != null ? quoteAsset : OnomyHubConstants.NOM_ASSET);
  const [currentPricePerBase, currentPricePerBaseStatus] = useDetailedBalance(quoteAsset, currentPriceFixed);
  const status = useAsyncStatuses([currentPriceStatus, currentPricePerBaseStatus]);
  return [currentPricePerBase, status];
}
async function includeOrderDetail({
  hub,
  onex,
  order,
  baseAsset,
  quoteAsset
}) {
  var _a;
  if (!hub)
    throw new Error("Hub is null");
  const direction = order.denomAsk === baseAsset.id ? "buy" : "sell";
  const baseAmount = {
    amount: direction === "sell" ? BigInt(order.amount) : BigInt(new BigNumber(order.rate[0]).times(order.amount).dividedBy(order.rate[1]).toFixed(0)),
    asset: baseAsset
  };
  const quoteAmountSimple = {
    amount: direction === "buy" ? BigInt(order.amount) : BigInt(new BigNumber(order.rate[0]).times(order.amount).dividedBy(order.rate[1]).toFixed(0)),
    asset: quoteAsset
  };
  const [price, amount, quoteAmount] = await Promise.all([
    (_a = onex == null ? void 0 : onex.getDisplayPriceFromRatio({
      baseDenom: baseAsset.id,
      baseAmountFixed: order.rate[direction === "buy" ? 0 : 1],
      quoteDenom: quoteAsset.id,
      quoteAmountFixed: order.rate[direction === "buy" ? 1 : 0]
    })) != null ? _a : Promise.resolve("0"),
    hub.balances.includeDetails(baseAmount),
    hub.balances.includeDetails(quoteAmountSimple)
  ]);
  const bnPrice = new BigNumber(price).shiftedBy(quoteAsset.meta.decimalPlaces);
  const pricePer = await hub.balances.includeDetails({
    amount: BigInt(bnPrice.toFixed(0)),
    asset: quoteAsset
  });
  return {
    ...order,
    amount,
    quoteAmount,
    price,
    pricePer,
    direction,
    baseAsset,
    quoteAsset
  };
}
function useOnexOrderHistory({
  baseAsset,
  quoteAsset,
  length = 10
}) {
  const { hub } = useOnomyHub();
  const onex = useOnexClient();
  const pair = useMemo(() => [baseAsset.id, quoteAsset.id].sort().join(","), [baseAsset.id, quoteAsset.id]);
  const cb = useCallback(async () => {
    var _a;
    try {
      const orders = hub && onex ? await onex.history({ pair, length: `${length}` }) : [];
      return await Promise.all(orders.map((order) => includeOrderDetail({ hub, onex, baseAsset, quoteAsset, order })));
    } catch (e) {
      if ((_a = e == null ? void 0 : e.message) == null ? void 0 : _a.match(/not found/))
        return [];
      throw e;
    }
  }, [baseAsset.id, quoteAsset.id, length, pair, hub, onex]);
  return useAsyncPoll(cb, []);
}
function useOnexMyActiveOrders({
  baseAsset,
  quoteAsset,
  length = 10
}) {
  const { hub } = useOnomyHub();
  const onex = useOnexClient();
  const pair = useMemo(() => [baseAsset.id, quoteAsset.id].sort().join(","), [baseAsset.id, quoteAsset.id]);
  const cb = useCallback(async () => {
    const orders = hub && onex ? await onex.ordersByOwner() : [];
    return Promise.all(orders.filter((order) => [order.denomAsk, order.denomBid].sort().join(",") === pair).map((order) => includeOrderDetail({ hub, onex, baseAsset, quoteAsset, order })));
  }, [baseAsset.id, quoteAsset.id, pair, length, hub, onex]);
  return useAsyncPoll(cb, []);
}
function useOnexMarketEstimate({ denomAsk = "usdt", denomBid = "anom", amountBid = "0" }) {
  const onex = useOnexClient();
  const cb = useCallback(async () => {
    const bnAmount = new BigNumber(amountBid);
    if (!onex || bnAmount.lte(0) || bnAmount.isNaN() || !bnAmount.isFinite())
      return "0";
    const resp = await onex.quote({
      denomAsk,
      denomBid,
      denomAmount: denomBid,
      amount: amountBid
    });
    return resp.amount;
  }, [denomAsk, denomBid, amountBid, onex]);
  return useAsyncPoll(cb, "0");
}
function useOnexOrderBook({
  baseAsset,
  quoteAsset,
  direction,
  orderType = "limit"
}) {
  const { hub } = useOnomyHub();
  const onex = useOnexClient();
  const cb = useCallback(async () => {
    const denomA = direction === "buy" ? baseAsset.id : quoteAsset.id;
    const denomB = direction === "sell" ? baseAsset.id : quoteAsset.id;
    const book = hub && onex ? await onex.book({ denomA, denomB, orderType }) : [];
    const res = await Promise.all(book.map((order) => includeOrderDetail({ hub, onex, baseAsset, quoteAsset, order })));
    return direction === "buy" ? res : res.reverse();
  }, [onex, baseAsset, quoteAsset, direction, hub, orderType]);
  return useAsyncPoll(cb, []);
}
function calculateDrops(pool, rawDrops) {
  const bnBaseMemberAmount = new BigNumber(pool.memberBase.balance);
  const bnQuoteMemberAmount = new BigNumber(pool.memberQuote.balance);
  const bnPoolDrops = new BigNumber(pool.drops);
  return rawDrops.map((drop) => {
    const bnProduct = new BigNumber(drop.product);
    const bnDrops = new BigNumber(drop.drops);
    const bnTotalBase = bnBaseMemberAmount.multipliedBy(bnDrops).dividedBy(bnPoolDrops);
    const bnTotalQuote = bnQuoteMemberAmount.multipliedBy(bnDrops).dividedBy(bnPoolDrops);
    const bnPrincipalBase = bnProduct.multipliedBy(bnBaseMemberAmount).dividedBy(bnQuoteMemberAmount).sqrt();
    const bnPrincipalQuote = bnProduct.multipliedBy(bnQuoteMemberAmount).dividedBy(bnBaseMemberAmount).sqrt();
    const bnRewardBase = bnTotalBase.minus(bnPrincipalBase);
    const bnRewardQuote = bnTotalQuote.minus(bnPrincipalQuote);
    return {
      ...drop,
      bnTotalBase,
      bnTotalQuote,
      bnPrincipalBase,
      bnPrincipalQuote,
      bnRewardBase,
      bnRewardQuote
    };
  });
}
async function includeDropsDetails(hub, pool, calculatedDrops) {
  if (!hub)
    throw new Error("Hub is null");
  return Promise.all(calculatedDrops.map(async (drop) => {
    const [totalBase, totalQuote, principalBase, principalQuote, rewardBase, rewardQuote] = await Promise.all([
      hub.balances.includeDetails({
        amount: BigInt(drop.bnTotalBase.toFixed(0)),
        asset: pool.baseAsset
      }),
      hub.balances.includeDetails({
        amount: BigInt(drop.bnTotalQuote.toFixed(0)),
        asset: pool.quoteAsset
      }),
      hub.balances.includeDetails({
        amount: BigInt(drop.bnPrincipalBase.toFixed(0)),
        asset: pool.baseAsset
      }),
      hub.balances.includeDetails({
        amount: BigInt(drop.bnPrincipalQuote.toFixed(0)),
        asset: pool.quoteAsset
      }),
      hub.balances.includeDetails({
        amount: BigInt(drop.bnRewardBase.toFixed(0)),
        asset: pool.baseAsset
      }),
      hub.balances.includeDetails({
        amount: BigInt(drop.bnRewardQuote.toFixed(0)),
        asset: pool.quoteAsset
      })
    ]);
    const principalFiat = principalBase.fiatValue + principalQuote.fiatValue;
    const rewardFiat = rewardBase.fiatValue + rewardQuote.fiatValue;
    const totalFiat = principalFiat + rewardFiat;
    return {
      ...drop,
      pool,
      totalBase,
      totalQuote,
      principalBase,
      principalQuote,
      rewardBase,
      rewardQuote,
      principalFiat,
      rewardFiat,
      totalFiat
    };
  }));
}
function useOnexMyDropsForPool(pool) {
  const { hub } = useOnomyHub();
  const onex = useOnexClient();
  const rawDropsCb = useCallback(async () => {
    var _a;
    if (!onex || !(pool == null ? void 0 : pool.pair))
      return [];
    try {
      return await onex.dropOwnerPair(pool.pair);
    } catch (e) {
      if ((_a = e == null ? void 0 : e.message) == null ? void 0 : _a.match(/not found/))
        return [];
      throw e;
    }
  }, [onex, pool == null ? void 0 : pool.pair]);
  const [rawDrops, rawDropsStatus] = useAsyncPoll(rawDropsCb, []);
  const calculatedDrops = useMemo(() => {
    if (!(pool == null ? void 0 : pool.pair))
      return [];
    return calculateDrops(pool, rawDrops);
  }, [rawDrops, pool == null ? void 0 : pool.pair, pool == null ? void 0 : pool.drops, pool == null ? void 0 : pool.memberBase.balance, pool == null ? void 0 : pool.memberQuote.balance]);
  const detailedDropsCb = useCallback(async () => {
    if (!hub || !pool)
      return [];
    return includeDropsDetails(hub, pool, calculatedDrops);
  }, [hub, calculatedDrops]);
  const [detailedDrops, detailedDropsStatus] = useAsyncPoll(detailedDropsCb, []);
  const status = useAsyncStatuses([rawDropsStatus, detailedDropsStatus]);
  return [detailedDrops, status];
}
function useOnexMyDrops() {
  const { hub } = useOnomyHub();
  const onex = useOnexClient();
  const dropPairsCb = useCallback(async () => {
    var _a, _b;
    try {
      return (_a = await (onex == null ? void 0 : onex.dropPairs())) != null ? _a : [];
    } catch (e) {
      if ((_b = e == null ? void 0 : e.message) == null ? void 0 : _b.match(/not found/))
        return [];
      throw e;
    }
  }, [onex]);
  const [allAssets, assetsStatus] = useSupportedAssetList();
  const [rawPairs, rawPairsStatus] = useAsyncPoll(dropPairsCb, []);
  const allDropsCb = useCallback(async () => {
    if (!onex)
      return [];
    const onexChain = onex.getBlockchain();
    const onexAssets = allAssets.filter((asset) => asset.chain.name === onexChain.name);
    if (!hub || onexAssets.length <= 0)
      return [];
    return Promise.all(rawPairs.map(async (pair) => {
      const [baseDenom, quoteDenom] = pair.split(",");
      const baseAsset = onexAssets.find((asset) => asset.id === baseDenom);
      const quoteAsset = onexAssets.find((asset) => asset.id === quoteDenom);
      if (!baseAsset)
        throw new Error(`baseAsset not found for ${baseDenom}`);
      if (!quoteAsset)
        throw new Error(`quoteAsset not found for ${quoteDenom}`);
      const [rawDrops, pool, memberBase, memberQuote] = await Promise.all([
        onex.dropOwnerPair(pair),
        onex.pool(pair),
        onex.member({ denomA: quoteDenom, denomB: baseDenom }),
        onex.member({ denomA: baseDenom, denomB: quoteDenom })
      ]);
      const detailedPool = await includePoolDetails({
        hub,
        onex,
        baseAsset,
        quoteAsset,
        pool,
        memberBase,
        memberQuote
      });
      const calculatedDrops = await calculateDrops(detailedPool, rawDrops);
      const drops = await includeDropsDetails(hub, detailedPool, calculatedDrops);
      const basePrincipalAmounts = drops.map((drop) => drop.principalBase);
      const quotePrincipalAmounts = drops.map((drop) => drop.principalQuote);
      const baseRewardAmounts = drops.map((drop) => drop.rewardBase);
      const quoteRewardAmounts = drops.map((drop) => drop.rewardQuote);
      const [basePrincipalRaw] = OnomyHubUtils.combineBalances(basePrincipalAmounts);
      const [quotePrincipalRaw] = OnomyHubUtils.combineBalances(quotePrincipalAmounts);
      const [baseRewardRaw] = OnomyHubUtils.combineBalances(baseRewardAmounts);
      const [quoteRewardRaw] = OnomyHubUtils.combineBalances(quoteRewardAmounts);
      const [baseTotalRaw] = OnomyHubUtils.combineBalances([basePrincipalRaw, baseRewardRaw]);
      const [quoteTotalRaw] = OnomyHubUtils.combineBalances([quotePrincipalRaw, quoteRewardRaw]);
      const [basePrincipal, quotePrincipal, baseReward, quoteReward, baseTotal, quoteTotal] = await Promise.all([
        hub.balances.includeDetails(basePrincipalRaw),
        hub.balances.includeDetails(quotePrincipalRaw),
        hub.balances.includeDetails(baseRewardRaw),
        hub.balances.includeDetails(quoteRewardRaw),
        hub.balances.includeDetails(baseTotalRaw),
        hub.balances.includeDetails(quoteTotalRaw)
      ]);
      const fiatPrincipal = basePrincipal.fiatValue + quotePrincipal.fiatValue;
      const fiatReward = baseReward.fiatValue + quoteReward.fiatValue;
      const fiatTotal = fiatPrincipal + fiatReward;
      return {
        pool: detailedPool,
        drops,
        basePrincipal,
        quotePrincipal,
        baseReward,
        quoteReward,
        baseTotal,
        quoteTotal,
        fiatPrincipal,
        fiatReward,
        fiatTotal
      };
    }));
  }, [hub, onex, rawPairs, allAssets]);
  return useAsyncPoll(allDropsCb, []);
}

function useExchangePairs() {
  const onex = useOnexClient();
  const [allAssets, assetsStatus] = useSupportedAssetList();
  const onexAssets = useMemo(() => {
    var _a;
    const chainId = (_a = onex == null ? void 0 : onex.getBlockchain().name) != null ? _a : "";
    const res = allAssets.filter((asset) => {
      return asset.chain.name === chainId && asset.id !== "anom" && asset.id !== "onex";
    });
    const anom = allAssets.find((asset) => {
      return asset.chain.name === chainId && asset.id === "anom";
    });
    return anom ? [anom, ...res] : res;
  }, [allAssets, onex]);
  const quoteAssets = onexAssets;
  const pairsByQuote = useMemo(() => {
    return onexAssets.reduce((res, quoteAsset) => {
      return {
        ...res,
        [quoteAsset.id]: {
          quoteAsset,
          pairs: onexAssets.filter((asset) => asset.id !== quoteAsset.id).map((baseAsset) => {
            return {
              quoteAsset,
              baseAsset,
              pair: [quoteAsset.id, baseAsset.id].sort().join(",")
            };
          })
        }
      };
    }, {});
  }, [onexAssets]);
  const loadingStatus = useAsyncStatuses([assetsStatus]);
  return {
    loadingStatus,
    assetsStatus,
    onexAssets,
    quoteAssets,
    pairsByQuote
  };
}

const BLANK_ONEX_HISTORY_FRAME = {
  startTime: "0",
  endTime: "0",
  startPrice: "0",
  maxPrice: "0",
  minPrice: "0",
  endPrice: "0",
  volume: "0"
};
const ONEX_HISTORICAL_DATA_QUERY = gql`
  query historicalFrames(
    $baseDenom: String!
    $quoteDenom: String!
    $frameType: String!
    $skipped: Int!
    $first: Int
    $from: Int
    $to: Int
  ) {
    historicalFrames(
      first: $first
      skip: $skipped
      where: { base: $baseDenom, quote: $quoteDenom, type: $frameType, startTime_lte: $to }
      orderBy: startTime
      orderDirection: desc
    ) {
      startTime
      endTime
      startPrice
      endPrice
      minPrice
      maxPrice
      volume
    }
  }
`;
class OnexHistoryFrames extends AbstractThrottledAsyncCache {
  constructor(apolloClient) {
    super();
    this.apolloClient = apolloClient;
  }
  cacheKey([{ baseDenom, quoteDenom, frameType, from, to, first, skipped }]) {
    return [baseDenom, quoteDenom, frameType, from, to, first, skipped].join("-");
  }
  async implementation({
    baseDenom,
    quoteDenom,
    frameType,
    from,
    to,
    first,
    skipped
  }) {
    try {
      const resp = await this.apolloClient.query({
        query: ONEX_HISTORICAL_DATA_QUERY,
        variables: {
          frameType,
          skipped,
          baseDenom,
          quoteDenom,
          from: from ? Math.round(from) : void 0,
          to: to ? Math.round(to) : Math.round(new Date().getTime() / 1e3),
          first
        }
      });
      const {
        data: { historicalFrames }
      } = resp;
      return historicalFrames;
    } catch (e) {
      console.error("OnexExchangeHistory error", e);
      throw e;
    }
  }
}

function PlaceholderErrorDisplay({ error: errorProp }) {
  useEffect(() => console.error("error", errorProp), [errorProp]);
  const error = (errorProp == null ? void 0 : errorProp.message) || errorProp;
  return /* @__PURE__ */ React.createElement(React.Fragment, null, `${error}`);
}

function useFindAsset(assetId, supportedAssets) {
  return React.useMemo(() => {
    var _a;
    return (_a = supportedAssets.find((a) => serializeAsset(a) === assetId)) != null ? _a : null;
  }, [supportedAssets, assetId]);
}
function useOnomyAppState(ecosystem) {
  var _a, _b;
  const { wallet } = useOnomyHub();
  const [supportedAssets, assetsStatus] = useSupportedAssetList();
  const supportedChains = useMemo(() => [
    OnomyHubConstants.ONOMY_CHAIN,
    ...ecosystem.onexChainInfo ? [OnomyHubConstants.ONEX_CHAIN] : [],
    ...ecosystem.ethWsUrl ? [OnomyHubConstants.ETH_CHAIN] : []
  ], [ecosystem.onexChainInfo, ecosystem.ethWsUrl]);
  const [myOnomyAddress, onomyAddressStatus] = useAddress(OnomyHubConstants.ONOMY_CHAIN);
  const [myOnexAddress, onexAddressStatus] = useAddress(OnomyHubConstants.ONEX_CHAIN);
  const onomyNomAsset = (_a = useFindAsset(`${OnomyHubConstants.ONOMY_CHAIN.name}_anom`, supportedAssets)) != null ? _a : OnomyHubConstants.BNOM_ASSET;
  const onexOnexAsset = (_b = useFindAsset(`${OnomyHubConstants.ONEX_CHAIN.name}_aonex`, supportedAssets)) != null ? _b : OnomyHubConstants.ONEX_ASSET;
  const onexNomAsset = useFindAsset(`${OnomyHubConstants.ONEX_CHAIN.name}_anom`, supportedAssets);
  const onexUsdtAsset = useFindAsset(`${OnomyHubConstants.ONEX_CHAIN.name}_ausdt`, supportedAssets);
  const onexUsdcAsset = useFindAsset(`${OnomyHubConstants.ONEX_CHAIN.name}_ausdc`, supportedAssets);
  const onexEthAsset = useFindAsset(`${OnomyHubConstants.ONEX_CHAIN.name}_wei`, supportedAssets);
  const { loadingStatus: pairsStatus, quoteAssets, pairsByQuote } = useExchangePairs();
  const isOnexAvailable = !!ecosystem.onexChainInfo;
  const onexHistoryFrames = useMemo(() => {
    if (!ecosystem.onexGraphqlEndpoint)
      return null;
    const apolloClient = new ApolloClient({
      uri: ecosystem.onexGraphqlEndpoint,
      cache: new InMemoryCache()
    });
    return new OnexHistoryFrames(apolloClient);
  }, [ecosystem.onexGraphqlEndpoint]);
  const myOnomyWallet = wallet.onomy.connectedWallet;
  const myOnexWallet = wallet.onex.connectedWallet;
  const loadingStatus = useAsyncStatuses([
    assetsStatus,
    pairsStatus,
    onomyAddressStatus,
    onexAddressStatus
  ]);
  const logout = useCallback(async () => {
    var _a2;
    await wallet.disconnect();
    (_a2 = window == null ? void 0 : window.location) == null ? void 0 : _a2.reload();
  }, [wallet]);
  const logoutOnomy = useCallback(async () => {
    var _a2;
    await wallet.onomy.disconnect();
    (_a2 = window.location) == null ? void 0 : _a2.reload();
  }, [wallet.onomy]);
  const logoutOnex = useCallback(async () => {
    var _a2;
    await wallet.onex.disconnect();
    (_a2 = window.location) == null ? void 0 : _a2.reload();
  }, [wallet.onex]);
  return {
    ecosystem,
    isOnexAvailable,
    loadingStatus,
    onexHistoryFrames,
    onomyChain: OnomyHubConstants.ONOMY_CHAIN,
    onexChain: OnomyHubConstants.ONEX_CHAIN,
    supportedChains,
    supportedAssets,
    onomyNomAsset,
    onexOnexAsset,
    onexNomAsset,
    onexUsdtAsset,
    onexUsdcAsset,
    onexEthAsset,
    unknownBalance: OnomyHubConstants.UNKNOWN_BALANCE,
    myOnomyAddress,
    myOnexAddress,
    myOnomyWallet,
    myOnexWallet,
    quoteAssets,
    pairsByQuote,
    logout,
    logoutOnomy,
    logoutOnex,
    isWalletLoaded: wallet.isLoaded,
    walletLoadError: wallet.error
  };
}
const OnomyAppContext = createContext(null);
function useOnomyApp() {
  const state = useContext(OnomyAppContext);
  if (!state)
    throw new Error("OnomyAppContext missing");
  return state;
}
function DEFAULT_ERROR_RENDER_FN$a(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function OnomyAppProvider({
  children,
  ecosystem,
  renderLoading,
  renderError = DEFAULT_ERROR_RENDER_FN$a
}) {
  const state = useOnomyAppState(ecosystem);
  if (state.loadingStatus.pending && renderLoading)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(OnomyAppContext.Provider, {
    value: state
  }, children);
}
function useAsset(idOrAsset) {
  const { supportedAssets } = useOnomyApp();
  const id = typeof idOrAsset === "string" ? idOrAsset : serializeAsset(idOrAsset);
  return useFindAsset(id, supportedAssets);
}

function useOnexDayTradeFrame({
  baseAsset,
  quoteAsset
}) {
  var _a, _b;
  const baseDenom = (_a = baseAsset == null ? void 0 : baseAsset.id) != null ? _a : "";
  const quoteDenom = (_b = quoteAsset == null ? void 0 : quoteAsset.id) != null ? _b : "";
  const { onexHistoryFrames } = useOnomyApp();
  const [lastPricePerBase, lastPricePerBaseStatus] = useOnexDetailedLastExecutionPrice({
    baseAsset,
    quoteAsset
  });
  const [[prevFrame = BLANK_ONEX_HISTORY_FRAME, priceFrame = BLANK_ONEX_HISTORY_FRAME], status] = useSubscribedValue(useCallback((cb) => {
    if (baseDenom === quoteDenom || !onexHistoryFrames || !baseDenom || !quoteDenom) {
      cb([]);
      return () => {
      };
    }
    return onexHistoryFrames.subscribe([
      {
        frameType: "Day",
        skipped: 0,
        baseDenom,
        quoteDenom,
        first: 2
      }
    ], cb);
  }, [onexHistoryFrames, baseDenom, quoteDenom]), [BLANK_ONEX_HISTORY_FRAME, BLANK_ONEX_HISTORY_FRAME]);
  const startPrice = parseFloat(priceFrame.startPrice);
  const endPrice = parseFloat(priceFrame.endPrice);
  const priceDiff = endPrice - startPrice;
  const priceChangeRatio = priceDiff / startPrice;
  const volumeDiff = new BigNumber(priceFrame.volume).minus(prevFrame.volume);
  const volumeDiffRatio = volumeDiff.dividedBy(prevFrame.volume).toNumber();
  const [volumeAmount, volumeAmountStatus] = useDetailedBalance(baseAsset, priceFrame.volume);
  const [volumeDiffAmount, volumeDiffAmountStatus] = useDetailedBalance(baseAsset, volumeDiff.toFixed(0));
  const loadingStatus = useAsyncStatuses([
    lastPricePerBaseStatus,
    status,
    volumeAmountStatus,
    volumeDiffAmountStatus
  ]);
  return [
    {
      ...priceFrame,
      loadingStatus,
      lastPricePerBase,
      lastPricePerBaseStatus,
      priceDiff,
      priceChangeRatio,
      volumeDiffRatio,
      volumeAmount,
      volumeAmountStatus,
      volumeDiffAmount,
      volumeDiffAmountStatus
    },
    status
  ];
}

const PERIOD_MAP = {
  "1": "Minute",
  "15": "QuarterHour",
  "60": "Hour",
  "480": "QuarterDay",
  "1D": "Day",
  "1W": "Week"
};
const ONEX_SYMBOL_INFO = {
  name: "bNOM",
  full_name: "bNOM",
  description: "bNOM",
  exchange: "ONEX",
  listed_exchange: "ONEX",
  format: "price",
  has_intraday: true,
  has_no_volume: true,
  has_seconds: false,
  has_ticks: false,
  has_daily: true,
  has_empty_bars: true,
  data_status: "streaming",
  original_currency_code: "ETH",
  currency_code: "ETH",
  supported_resolutions: Object.keys(PERIOD_MAP),
  intraday_multipliers: Object.keys(PERIOD_MAP),
  ticker: "bNOM",
  timezone: "Etc/UTC",
  type: "crypto",
  session: "24x7",
  minmov: 1,
  pricescale: 1e5
};
class OnexTradingViewData {
  constructor(onexHistoryFrames, baseDenom, quoteDenom) {
    this.onexHistoryFrames = onexHistoryFrames;
    this.barSubs = {};
    this.lastTick = {};
    this.baseDenom = baseDenom;
    this.quoteDenom = quoteDenom;
    this.onBlockMined = this.onBlockMined.bind(this);
    this.timer = setInterval(this.onBlockMined.bind(this), 5e3);
  }
  async onBlockMined() {
    const subs = Object.values(this.barSubs);
    for (const { symbolInfo, resolution, onTick } of subs) {
      this.getBars(symbolInfo, resolution, { countBack: 1, from: 0, to: 0, firstDataRequest: false }, (res) => {
        const last = res.slice().pop();
        if (last && (!this.lastTick[resolution] || this.lastTick[resolution] < last.time)) {
          onTick(last);
        }
      }, (err) => console.error(err));
    }
  }
  destroy() {
    clearInterval(this.timer);
  }
  onReady(cb) {
    setTimeout(() => cb({
      exchanges: [
        {
          value: ONEX_SYMBOL_INFO.exchange,
          name: ONEX_SYMBOL_INFO.exchange,
          desc: ONEX_SYMBOL_INFO.exchange
        }
      ],
      symbols_types: [
        {
          value: ONEX_SYMBOL_INFO.type,
          name: ONEX_SYMBOL_INFO.type
        }
      ],
      supported_resolutions: ONEX_SYMBOL_INFO.supported_resolutions,
      currency_codes: ["ETH"],
      units: {},
      supports_marks: false,
      supports_timescale_marks: false,
      supports_time: true
    }), 0);
  }
  async searchSymbols(userInput, exchange, symbolType, onResult) {
    onResult([
      {
        symbol: ONEX_SYMBOL_INFO.ticker,
        full_name: ONEX_SYMBOL_INFO.name,
        description: ONEX_SYMBOL_INFO.description,
        exchange: ONEX_SYMBOL_INFO.exchange,
        ticker: ONEX_SYMBOL_INFO.ticker,
        type: ONEX_SYMBOL_INFO.type
      }
    ]);
  }
  resolveSymbol(symbolName, onSuccess) {
    setTimeout(() => onSuccess(ONEX_SYMBOL_INFO), 0);
  }
  async getBars(symbolInfo, resolution, periodParams, onResult, onError) {
    var _a;
    try {
      const frames = await this.onexHistoryFrames.get({
        frameType: PERIOD_MAP[resolution] || "Hour",
        skipped: 0,
        baseDenom: this.baseDenom,
        quoteDenom: this.quoteDenom,
        from: periodParams.from,
        to: (_a = periodParams.to) != null ? _a : void 0,
        first: Math.min(1e3, periodParams.countBack)
      });
      const bars = frames.slice().reverse().map((frame) => {
        const time = parseInt(frame.startTime, 10) * 1e3;
        if (!this.lastTick[resolution] || this.lastTick[resolution] <= time) {
          this.lastTick[resolution] = time;
        } else {
        }
        return {
          time,
          open: frame.startPrice,
          high: frame.maxPrice,
          low: frame.minPrice,
          close: frame.endPrice,
          volume: frame.volume
        };
      });
      if (bars.length > 0) {
        onResult(bars, { noData: false });
      } else {
        onResult([], { noData: true });
      }
    } catch (e) {
      console.error(e);
      onError(`${e}`);
    }
  }
  subscribeBars(symbolInfo, resolution, onTick, listenerGuid, onResetCacheNeededCallback) {
    this.barSubs[listenerGuid] = {
      symbolInfo,
      resolution,
      onTick,
      listenerGuid,
      onResetCacheNeededCallback
    };
  }
  unsubscribeBars(listenerGuid) {
    delete this.barSubs[listenerGuid];
  }
}
function useOnexTradingViewData(baseDenom, quoteDenom) {
  const { onexHistoryFrames } = useOnomyApp();
  try {
    return useMemo(() => {
      if (!onexHistoryFrames)
        return null;
      return new OnexTradingViewData(onexHistoryFrames, baseDenom, quoteDenom);
    }, [onexHistoryFrames, baseDenom, quoteDenom]);
  } catch (e) {
    console.warn("useOnexTradeData error", e);
    return null;
  }
}

const ONEX_ORDER_BOOKS_QUERY = gql`
  query orderBooks($base: String!, $quote: String!) {
    orderBooks(where: { base: $base, quote: $quote }) {
      id
      total
      bins
    }
  }
`;
const ONEX_ORDER_BOOK_INCREMENTS_QUERY = gql`
  query bookIncrements($book: String!, $bin: BigDecimal!) {
    bookIncrements(where: { book: $book, bin: $bin }, orderBy: place, orderDirection: desc) {
      id
      place
      orders
      amount
    }
  }
`;
function useOrderBookBins({
  apolloClient,
  baseAsset,
  quoteAsset
}) {
  const cb = useCallback(async () => {
    var _a, _b;
    const resp = await apolloClient.query({
      query: ONEX_ORDER_BOOKS_QUERY,
      variables: {
        base: baseAsset.id,
        quote: quoteAsset.id
      }
    });
    const {
      data: { orderBooks: books }
    } = resp;
    const normalized = books.map((book) => {
      return {
        ...book,
        bins: book.bins.map((binId) => binId.split("-").pop())
      };
    });
    const bins = ((_b = (_a = normalized[0]) == null ? void 0 : _a.bins) != null ? _b : []).sort((a, b) => {
      return parseFloat(a) - parseFloat(b);
    });
    return bins;
  }, [apolloClient, baseAsset.id, quoteAsset.id]);
  return useAsyncPoll(cb, []);
}
function useOrderBookIncrements({
  apolloClient,
  baseAsset,
  quoteAsset,
  bin,
  direction
}) {
  const { hub } = useOnomyHub();
  const cb = useCallback(async () => {
    if (!bin || !hub)
      return [];
    const book = `${baseAsset.id}-${quoteAsset.id}-${direction}`;
    const resp = await apolloClient.query({
      query: ONEX_ORDER_BOOK_INCREMENTS_QUERY,
      variables: {
        book,
        bin
      }
    });
    let {
      data: { bookIncrements = [] }
    } = resp;
    let totalBase = new BigNumber(0);
    try {
      if (direction === "sell")
        bookIncrements = bookIncrements.slice().reverse();
    } catch (e) {
      console.error(e);
    }
    const increments = bookIncrements.map((raw) => {
      const bnPlace = new BigNumber(raw.place);
      const bnPricePer = bnPlace.shiftedBy(quoteAsset.meta.decimalPlaces);
      const bnQuoteAmount = bnPlace.times(raw.amount);
      totalBase = totalBase.plus(raw.amount);
      return {
        uid: raw.id,
        place: raw.place,
        amountRaw: raw.amount,
        orders: raw.orders,
        amount: {
          amount: BigInt(raw.amount),
          asset: baseAsset
        },
        totalAmount: {
          amount: BigInt(totalBase.toFixed(0)),
          asset: baseAsset
        },
        quoteAmount: {
          amount: BigInt(bnQuoteAmount.toFixed(0)),
          asset: quoteAsset
        },
        pricePer: {
          amount: BigInt(bnPricePer.toFixed(0)),
          asset: quoteAsset
        }
      };
    });
    if (direction === "sell")
      increments.reverse();
    const detailedIncrements = await Promise.all(increments.map(async (inc) => {
      const [amount, totalAmount, quoteAmount, pricePer] = await hub.balances.includeDetailsMultiple([
        inc.amount,
        inc.totalAmount,
        inc.quoteAmount,
        inc.pricePer
      ]);
      return {
        ...inc,
        percentage: new BigNumber(totalAmount.amount.toString()).dividedBy(totalBase).times(100).toNumber(),
        amount,
        totalAmount,
        quoteAmount,
        pricePer
      };
    }));
    return detailedIncrements;
  }, [hub, apolloClient, baseAsset.id, quoteAsset.id, bin, direction]);
  return useAsyncPoll(cb, []);
}
function useIndexedOrderBookForm({
  baseAsset,
  quoteAsset
}) {
  var _a;
  const { ecosystem } = useOnomyApp();
  const apolloClient = useMemo(() => {
    return new ApolloClient({
      uri: ecosystem.onexGraphqlEndpoint,
      cache: new InMemoryCache()
    });
  }, [ecosystem.onexGraphqlEndpoint]);
  const [orderBins, orderBinsStatus] = useOrderBookBins({
    apolloClient,
    baseAsset,
    quoteAsset
  });
  const binOptions = useMemo(() => orderBins.map((bin) => {
    const val = parseFloat(bin);
    return {
      value: bin,
      label: val > 1 ? val.toLocaleString() : val.toString()
    };
  }), [orderBins]);
  const [selectedBin, setSelectedBin] = useState(void 0);
  useMemo(() => {
    if (binOptions[0] && typeof selectedBin === "undefined") {
      setSelectedBin(binOptions[0].value);
    }
  }, [(_a = binOptions[0]) == null ? void 0 : _a.value]);
  const [bookBuy, bookBuyStatus] = useOrderBookIncrements({
    apolloClient,
    baseAsset,
    quoteAsset,
    bin: selectedBin != null ? selectedBin : "",
    direction: "buy"
  });
  const [bookSell, bookSellStatus] = useOrderBookIncrements({
    apolloClient,
    baseAsset,
    quoteAsset,
    bin: selectedBin != null ? selectedBin : "",
    direction: "sell"
  });
  const loadingStatus = useAsyncStatuses([orderBinsStatus, bookBuyStatus, bookSellStatus]);
  const chartData = useMemo(() => {
    const bids = bookBuy.map((item) => {
      return {
        side: "buy",
        price: parseFloat(item.pricePer.amountFormatted),
        amount: parseFloat(item.amount.amountFormatted)
      };
    });
    const asks = bookSell.map((item) => {
      return {
        side: "sell",
        price: parseFloat(item.pricePer.amountFormatted),
        amount: parseFloat(item.amount.amountFormatted)
      };
    });
    return {
      market: `${baseAsset.meta.symbol}/${quoteAsset.meta.symbol}`,
      bids,
      asks
    };
  }, [bookBuy, bookSell, baseAsset.meta.symbol, quoteAsset.meta.symbol]);
  return {
    loadingStatus,
    orderBinsStatus,
    bookBuyStatus,
    bookSellStatus,
    binOptions,
    selectedBin,
    bookBuy,
    bookSell,
    chartData,
    setSelectedBin
  };
}

const PROPOSAL_STATUS = {
  PROPOSAL_STATUS_UNSPECIFIED: "Unspecified",
  PROPOSAL_STATUS_DEPOSIT_PERIOD: "Deposit Period",
  PROPOSAL_STATUS_VOTING_PERIOD: "Voting Period",
  PROPOSAL_STATUS_PASSED: "Passed",
  PROPOSAL_STATUS_REJECTED: "Rejected",
  PROPOSAL_STATUS_FAILED: "Failed",
  UNRECOGNIZED: "Unrecognized"
};
const PROPOSAL_TYPE = {
  ParameterChangeProposal: {
    name: "Parameter Change"
  },
  CommunityPoolSpendProposal: {
    name: "Community Pool Spend"
  },
  FundAccountProposal: {
    name: "DAO Fund Account"
  },
  SoftwareUpgradeProposal: {
    name: "Software Upgrade"
  }
};
function isUndefined(index) {
  return typeof index === "undefined";
}
function getProposalType(index) {
  var _a;
  const undefinedType = {
    name: "Unknown"
  };
  return isUndefined(index) ? undefinedType : (_a = PROPOSAL_TYPE[index]) != null ? _a : {
    name: index == null ? void 0 : index.split("/").pop()
  };
}
function getProposalStatus(index) {
  return isUndefined(index) ? "Unknown" : PROPOSAL_STATUS[index];
}
function renderRemainingDays(endDate) {
  if (!endDate)
    return "No end date specified";
  const timeDifference = (Date.parse(endDate.toString()) - Date.now()) / 1e3 / 60 / 60 / 24;
  if (timeDifference < 0)
    return "Voting Ended";
  if (timeDifference < 1)
    return "Less than 1 day left";
  if (timeDifference < 2)
    return "1 day left";
  return `${Math.floor(timeDifference)} days left`;
}
function calculateVotesChartData(proposal) {
  var _a, _b, _c, _d, _e, _f, _g, _h, _i;
  return ((_a = proposal == null ? void 0 : proposal.results) == null ? void 0 : _a.total) ? [
    new BigNumber((_c = (_b = proposal.results) == null ? void 0 : _b.yes) != null ? _c : 0).dividedBy(proposal.results.total).times(100).toNumber(),
    new BigNumber((_e = (_d = proposal.results) == null ? void 0 : _d.no) != null ? _e : 0).plus((_g = (_f = proposal.results) == null ? void 0 : _f.noWithVeto) != null ? _g : 0).dividedBy(proposal.results.total).times(100).toNumber(),
    new BigNumber((_i = (_h = proposal.results) == null ? void 0 : _h.abstain) != null ? _i : 0).dividedBy(proposal.results.total).times(100).toNumber()
  ].map((x) => {
    if (Number.isNaN(x))
      return 0;
    return x;
  }) : [0, 0, 0];
}

var governanceUtils = /*#__PURE__*/Object.freeze({
  __proto__: null,
  PROPOSAL_STATUS: PROPOSAL_STATUS,
  PROPOSAL_TYPE: PROPOSAL_TYPE,
  isUndefined: isUndefined,
  getProposalType: getProposalType,
  getProposalStatus: getProposalStatus,
  renderRemainingDays: renderRemainingDays,
  calculateVotesChartData: calculateVotesChartData
});

function PlaceholderLoadingIndicator({ text = "Loading..." }) {
  return /* @__PURE__ */ React.createElement(React.Fragment, null, text);
}

function useMnemonicWalletState() {
  const { wallet } = useOnomyHub();
  const [generatedMnemonic, setGeneratedMnemonic] = useState("");
  const connectWallet = useCallback(async () => {
    return Promise.all([
      wallet.onomy.connect(wallet.onomy.wallets[0]),
      wallet.onex.connect(wallet.onex.wallets[0])
    ]);
  }, [wallet]);
  const saveMnemonic = useCallback(async (mnemonic) => {
    await wallet.doSave(mnemonic.toLowerCase());
    await connectWallet();
    setGeneratedMnemonic("");
  }, [wallet, connectWallet]);
  const getMnemonic = useCallback(async (password) => wallet.getMnemonic(password), [wallet]);
  const generateMnemonic = useCallback(async () => {
    if (!wallet.isMissing && !generatedMnemonic)
      return;
    const created = await wallet.doCreate();
    setGeneratedMnemonic(created);
  }, [wallet, generatedMnemonic]);
  const eraseMnemonic = useCallback(async () => {
    await wallet.doErase();
  }, [wallet]);
  useEffect(() => {
    (async () => {
      try {
        if (wallet.isLoaded && !wallet.isMissing)
          await connectWallet();
      } catch (e) {
        console.error("wallet load error", e);
      }
    })();
  }, [wallet.isLoaded]);
  return {
    error: wallet.error,
    isWalletLoaded: !!(wallet.isLoaded || generatedMnemonic),
    isMnemonicSaved: !wallet.isMissing && !generatedMnemonic,
    generatedMnemonic,
    generateMnemonic,
    getMnemonic,
    checkMnemonic: wallet.checkMnemonic,
    saveMnemonic,
    eraseMnemonic
  };
}
const MnemonicWalletContext = createContext(null);
function useMnemonicWallet() {
  const state = useContext(MnemonicWalletContext);
  if (!state)
    throw new Error("Missing MnemonicWalletContext");
  return state;
}
function defaultRenderError$6(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function MnemonicWalletProvider({
  children,
  renderLoading,
  renderError = defaultRenderError$6
}) {
  const state = useMnemonicWalletState();
  if (state.error)
    return renderError(state.error);
  if (!state.isWalletLoaded && renderLoading)
    return renderLoading();
  return /* @__PURE__ */ React.createElement(MnemonicWalletContext.Provider, {
    value: state
  }, children);
}

function OnomyCoreProvider({
  children,
  createBackendFn,
  config = "DEVNET",
  overrides,
  renderLoading,
  renderError
}) {
  const ecosystem = useMemo(() => {
    return setOnomyEcosystemConfig(config, overrides);
  }, []);
  const backend = useMemo(createBackendFn, []);
  return /* @__PURE__ */ React.createElement(OnomyHubProvider, {
    backend,
    onomyChainInfo: ecosystem.onomyChainInfo,
    onexChainInfo: ecosystem.onexChainInfo,
    ethWsUrl: ecosystem.ethWsUrl,
    nomContractAddress: ecosystem.bNomContractAddress,
    bondContractAddress: ecosystem.bondContractAddress,
    gravityContractAddress: ecosystem.gravityContractAddress,
    wEthUsdtUniswapPoolAddress: ecosystem.wEthUsdUniswapPoolAddress
  }, /* @__PURE__ */ React.createElement(OnomyAppProvider, {
    ecosystem,
    renderLoading,
    renderError
  }, children));
}

async function includeVestingDetails({
  client,
  hub,
  vesting
}) {
  if (!hub)
    throw new Error("Hub is null");
  if (!vesting)
    return null;
  const now = new Date().getTime() / 1e3;
  const startTime = vesting.startTime.toNumber();
  const chain = client.getBlockchain();
  function coinDetails(coin) {
    return hub.balances.includeDetails({
      amount: BigInt(coin.amount),
      asset: {
        id: coin.denom,
        chain
      }
    });
  }
  async function getBaseVestingAccount() {
    const [delegatedFree, delegatedVesting, originalVesting] = await Promise.all([
      Promise.all(vesting.baseVestingAccount.delegatedFree.map(coinDetails)),
      Promise.all(vesting.baseVestingAccount.delegatedVesting.map(coinDetails)),
      Promise.all(vesting.baseVestingAccount.originalVesting.map(coinDetails))
    ]);
    return {
      ...vesting,
      delegatedFree,
      delegatedVesting,
      originalVesting
    };
  }
  async function getVestingPeriods() {
    var _a, _b;
    let totalWait = 0;
    const periods = (_a = vesting == null ? void 0 : vesting.vestingPeriods.map((period) => {
      totalWait += period.length.toNumber();
      const unlocksAt = startTime + totalWait;
      const isUnlocked = now >= unlocksAt;
      return {
        ...period,
        totalWait,
        unlocksAt,
        isUnlocked
      };
    })) != null ? _a : [];
    return Promise.all((_b = periods.map(async (period) => {
      const amounts = await Promise.all(period.amount.map(coinDetails));
      return {
        ...period,
        amounts
      };
    })) != null ? _b : []);
  }
  const [baseVestingAccount, vestingPeriods] = await Promise.all([
    getBaseVestingAccount(),
    getVestingPeriods()
  ]);
  const allLockedRaw = vestingPeriods.reduce((res, period) => {
    if (!period.isUnlocked)
      return res;
    return [...res, ...period.amounts];
  }, []);
  const allLockedCombined = OnomyHub.Utils.combineBalances(allLockedRaw);
  const allLocked = await Promise.all(allLockedCombined.map((amount) => hub.balances.includeDetails(amount)));
  const zero = BigInt(0);
  const allLockedStakedRaw = [];
  const allLockedUnstakedRaw = allLocked.map((amt) => {
    var _a, _b, _c, _d;
    const assetId = serializeAsset(amt.asset);
    const delegatedVesting = (_b = (_a = baseVestingAccount.delegatedVesting.find((a) => serializeAsset(a.asset) === assetId)) == null ? void 0 : _a.amount) != null ? _b : zero;
    const delegatedFree = (_d = (_c = baseVestingAccount.delegatedFree.find((a) => serializeAsset(a.asset) === assetId)) == null ? void 0 : _c.amount) != null ? _d : zero;
    let staked = delegatedVesting - delegatedFree;
    if (staked < zero)
      staked = zero;
    let amount = amt.amount - staked;
    if (amount < zero)
      amount = zero;
    let stakedAmount = amt.amount - amount;
    if (stakedAmount < zero)
      stakedAmount = zero;
    allLockedStakedRaw.push({
      asset: amt.asset,
      amount: stakedAmount
    });
    return {
      asset: amt.asset,
      amount
    };
  });
  const [allLockedStaked, allLockedUnstaked] = await Promise.all([
    Promise.all(allLockedStakedRaw.map((amount) => hub.balances.includeDetails(amount))),
    Promise.all(allLockedUnstakedRaw.map((amount) => hub.balances.includeDetails(amount)))
  ]);
  const allLockedFiatValue = allLocked.reduce((res, item) => res + item.fiatValue, 0);
  const allLockedStakedFiatValue = allLocked.reduce((res, item) => res + item.fiatValue, 0);
  const allLockedUnstakedFiatValue = allLocked.reduce((res, item) => res + item.fiatValue, 0);
  return {
    ...vesting,
    chain,
    baseVestingAccount,
    vestingPeriods,
    allLocked,
    allLockedStaked,
    allLockedUnstaked,
    allLockedFiatValue,
    allLockedStakedFiatValue,
    allLockedUnstakedFiatValue
  };
}
function useVesting() {
  const { hub } = useOnomyHub();
  const cb = useCallback(async () => {
    if (!hub)
      return null;
    const clients = [hub.chains.onomy.client];
    if (hub.chains.onex)
      clients.push(hub.chains.onex.client);
    const vestingDataRaw = await Promise.all(clients.map(async (client) => {
      return includeVestingDetails({
        client,
        hub,
        vesting: await (async () => {
          var _a;
          try {
            return await client.getVesting();
          } catch (e) {
            if ((_a = e == null ? void 0 : e.message) == null ? void 0 : _a.match(/not found/))
              return null;
            throw e;
          }
        })()
      });
    }));
    const vestingData = vestingDataRaw.filter((x) => x !== null);
    if (!vestingData.length)
      return null;
    const allLockedRaw = vestingData.reduce((res, item) => {
      var _a;
      return [...res, ...(_a = item == null ? void 0 : item.allLocked) != null ? _a : []];
    }, []);
    const allLockedStakedRaw = vestingData.reduce((res, item) => {
      var _a;
      return [...res, ...(_a = item == null ? void 0 : item.allLockedStaked) != null ? _a : []];
    }, []);
    const allLockedUnstakedRaw = vestingData.reduce((res, item) => {
      var _a;
      return [...res, ...(_a = item == null ? void 0 : item.allLockedUnstaked) != null ? _a : []];
    }, []);
    const allLockedCombined = OnomyHubUtils.combineBalances(allLockedRaw);
    const allLockedStakedCombined = OnomyHubUtils.combineBalances(allLockedStakedRaw);
    const allLockedUnstakedCombined = OnomyHubUtils.combineBalances(allLockedUnstakedRaw);
    const [allLocked, allLockedStaked, allLockedUnstaked] = await Promise.all([
      Promise.all(allLockedCombined.map((amount) => hub.balances.includeDetails(amount))),
      Promise.all(allLockedStakedCombined.map((amount) => hub.balances.includeDetails(amount))),
      Promise.all(allLockedUnstakedCombined.map((amount) => hub.balances.includeDetails(amount)))
    ]);
    const allLockedFiatValue = allLocked.reduce((res, item) => res + item.fiatValue, 0);
    const allLockedStakedFiatValue = allLockedStaked.reduce((res, item) => res + item.fiatValue, 0);
    const allLockedUnstakedFiatValue = allLockedUnstaked.reduce((res, item) => res + item.fiatValue, 0);
    return {
      allLocked,
      allLockedStaked,
      allLockedUnstaked,
      allLockedFiatValue,
      allLockedStakedFiatValue,
      allLockedUnstakedFiatValue,
      vestingData
    };
  }, [hub]);
  return useAsyncPoll(cb, null);
}

function useOnomyDashboardState() {
  const {
    onomyNomAsset,
    onexOnexAsset,
    onexNomAsset,
    onexEthAsset,
    onexUsdtAsset,
    loadingStatus: appLoadingStatus
  } = useOnomyApp();
  const holdings = useHoldings();
  const [vesting, vestingStatus] = useVesting();
  const [myLiquidity, myLiquidityStatus] = useOnexMyDrops();
  const [delegations, delegationsStatus] = useDelegations();
  const oneNomFixed = useMemo(() => new BigNumber("1").shiftedBy(onomyNomAsset.meta.decimalPlaces).toFixed(0), [onomyNomAsset.meta.decimalPlaces]);
  const [oneNomCoin] = useDetailedBalance(onomyNomAsset, oneNomFixed);
  const { principalTotalFiat, rewardTotalFiat, liquidityTotalFiat } = useMemo(() => {
    return myLiquidity.reduce((res, item) => {
      res.principalTotalFiat += item.fiatPrincipal;
      res.rewardTotalFiat += item.fiatReward;
      res.liquidityTotalFiat += item.fiatTotal;
      return res;
    }, {
      principalTotalFiat: 0,
      rewardTotalFiat: 0,
      liquidityTotalFiat: 0
    });
  }, [myLiquidity]);
  const onexBalancesRaw = useMemo(() => holdings.balances.filter((x) => x.asset.chain.name === OnomyHubConstants.ONEX_CHAIN_ID), [holdings.balances]);
  const mainBalancesRaw = useMemo(() => holdings.balances.filter((x) => x.asset.chain.name !== OnomyHubConstants.ONEX_CHAIN_ID), [holdings.balances]);
  const [onexBalances, onexBalancesStatus] = useDetailedBalances(onexBalancesRaw);
  const [mainBalances, mainBalancesStatus] = useDetailedBalances(mainBalancesRaw);
  const onexBalancesTotalFiat = useFiatTotal(onexBalances);
  const mainBalancesTotalFiat = useFiatTotal(mainBalances);
  const mainGrandTotalFiat = holdings.stakedTotalFiat + holdings.rewardTotalFiat + mainBalancesTotalFiat;
  const [nomUsdtDay, nomUsdtDayStatus] = useOnexDayTradeFrame({
    baseAsset: onexNomAsset,
    quoteAsset: onexUsdtAsset
  });
  const [nomEthDay, nomUsdtEthStatus] = useOnexDayTradeFrame({
    baseAsset: onexNomAsset,
    quoteAsset: onexEthAsset
  });
  const [onexNomBurned, onexNomBurnedStatus] = useOnexBurned();
  const onexStaked = useAssetInBalances(onexOnexAsset, holdings.stakedBalances);
  const onexRewards = useAssetInBalances(onexOnexAsset, holdings.rewardBalances);
  const onexUnbonding = useAssetInBalances(onexOnexAsset, holdings.unbondingBalances);
  const onexGrandTotalFiat = onexBalancesTotalFiat + liquidityTotalFiat + onexStaked.fiatValue + onexRewards.fiatValue + onexUnbonding.fiatValue;
  const grandTotalFiat = mainGrandTotalFiat + onexGrandTotalFiat;
  const loadingStatus = useAsyncStatuses([
    appLoadingStatus,
    holdings.loadingStatus,
    vestingStatus,
    nomUsdtDayStatus,
    nomUsdtEthStatus,
    myLiquidityStatus,
    delegationsStatus,
    onexBalancesStatus,
    mainBalancesStatus,
    onexNomBurnedStatus
  ]);
  return {
    holdings,
    vesting,
    myLiquidity,
    delegations,
    nomUsdtDay,
    nomEthDay,
    onexNomBurned,
    onexBalances,
    mainBalances,
    oneNomCoin,
    onexStaked,
    onexRewards,
    onexUnbonding,
    principalTotalFiat,
    rewardTotalFiat,
    liquidityTotalFiat,
    onexBalancesTotalFiat,
    mainBalancesTotalFiat,
    mainGrandTotalFiat,
    onexGrandTotalFiat,
    grandTotalFiat,
    loadingStatus
  };
}
const OnomyDashboardContext = createContext(null);
function useOnomyDashboard() {
  const state = useContext(OnomyDashboardContext);
  if (!state)
    throw new Error("OnomyDashboardContext missing");
  return state;
}
function DEFAULT_ERROR_RENDER_FN$9(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function OnomyDashboardProvider({
  children,
  renderLoading,
  renderError = DEFAULT_ERROR_RENDER_FN$9
}) {
  const state = useOnomyDashboardState();
  if (state.loadingStatus.pending && renderLoading)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(OnomyDashboardContext.Provider, {
    value: state
  }, children);
}
function useFindBalance(assetId, balances) {
  return React.useMemo(() => {
    var _a;
    return (_a = balances.find((a) => serializeAsset(a.asset) === assetId)) != null ? _a : OnomyHubConstants.UNKNOWN_BALANCE;
  }, [balances, assetId]);
}
function useAvailableBalance(idOrAsset) {
  const { holdings } = useOnomyDashboard();
  const id = typeof idOrAsset === "string" ? idOrAsset : serializeAsset(idOrAsset);
  return useFindBalance(id, holdings.balances);
}
function useStakedBalance(idOrAsset) {
  const { holdings } = useOnomyDashboard();
  const id = typeof idOrAsset === "string" ? idOrAsset : serializeAsset(idOrAsset);
  return useFindBalance(id, holdings.stakedBalances);
}
function useRewardBalance(idOrAsset) {
  const { holdings } = useOnomyDashboard();
  const id = typeof idOrAsset === "string" ? idOrAsset : serializeAsset(idOrAsset);
  return useFindBalance(id, holdings.rewardBalances);
}
function useUnstakingBalance(idOrAsset) {
  const { holdings } = useOnomyDashboard();
  const id = typeof idOrAsset === "string" ? idOrAsset : serializeAsset(idOrAsset);
  return useFindBalance(id, holdings.unbondingBalances);
}
function useAssetDelegations(idOrAsset) {
  const { delegations } = useOnomyDashboard();
  const id = typeof idOrAsset === "string" ? idOrAsset : serializeAsset(idOrAsset);
  return React.useMemo(() => delegations.filter((a) => serializeAsset(a.delegated.asset) === id), [delegations, id]);
}
function useChainDelegations(idOrChain) {
  const { delegations } = useOnomyDashboard();
  const id = typeof idOrChain === "string" ? idOrChain : idOrChain.name;
  return React.useMemo(() => delegations.filter((a) => a.delegated.asset.chain.name === id), [delegations, id]);
}

function useTransferAssetState(asset) {
  const form = useTransferAssetForm(asset);
  return form;
}
const TransferAssetContext = createContext(null);
function useTransferAsset() {
  const state = useContext(TransferAssetContext);
  if (!state)
    throw new Error("TransferAssetContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN$8() {
  return /* @__PURE__ */ React.createElement(PlaceholderLoadingIndicator, {
    text: "Loading transfer form"
  });
}
function DEFAULT_ERROR_RENDER_FN$8(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function TransferAssetProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN$8,
  renderError = DEFAULT_ERROR_RENDER_FN$8,
  asset = null
}) {
  const state = useTransferAssetState(asset);
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(TransferAssetContext.Provider, {
    value: state
  }, children);
}

const ClaimRewardContext = createContext(null);
function useClaimReward() {
  const state = useContext(ClaimRewardContext);
  if (!state)
    throw new Error("ClaimRewardContext missing");
  return state;
}
function defaultRenderError$5(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function ClaimRewardProvider({
  children,
  renderLoading,
  renderError = defaultRenderError$5,
  ...props
}) {
  const state = useClaimRewardForm(props);
  if (renderLoading && (state.loadingStatus.pending || !state.delegation))
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(ClaimRewardContext.Provider, {
    value: state
  }, children);
}

function useDelegateAssetState(props) {
  const { undelegate: undelegateProp } = props != null ? props : {};
  const [undelegate, setUndelegate] = useState(undelegateProp);
  const toggleUndelegate = useCallback(() => setUndelegate((u) => !u), []);
  const form = useDelegateAssetForm({ ...props, undelegate });
  return {
    ...form,
    undelegate,
    setUndelegate,
    toggleUndelegate
  };
}
const DelegateAssetContext = createContext(null);
function useDelegateAsset() {
  const state = useContext(DelegateAssetContext);
  if (!state)
    throw new Error("DelegateAssetContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN$7() {
  return /* @__PURE__ */ React.createElement(PlaceholderLoadingIndicator, {
    text: "Loading transfer form"
  });
}
function DEFAULT_ERROR_RENDER_FN$7(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function DelegateAssetProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN$7,
  renderError = DEFAULT_ERROR_RENDER_FN$7,
  ...props
}) {
  const state = useDelegateAssetState(props);
  if (state.loadingStatus.pending || !state.delegation)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(DelegateAssetContext.Provider, {
    value: state
  }, children);
}

const MoveDelegationContext = createContext(null);
function useMoveDelegation() {
  const state = useContext(MoveDelegationContext);
  if (!state)
    throw new Error("Missing MoveDelegationContext");
  return state;
}
function defaultRenderError$4(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function MoveDelegationProvider({
  children,
  renderLoading,
  renderError = defaultRenderError$4,
  ...props
}) {
  const state = useMoveDelegationForm(props);
  if (renderLoading && (state.loadingStatus.pending || !state.srcDelegation))
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(MoveDelegationContext.Provider, {
    value: state
  }, children);
}

function useValidatorDetailState({ validatorAddress }) {
  const [delegation, delegationStatus] = useDelegation(validatorAddress);
  const claimRewards = useClaimRewardForm({ validatorAddress });
  const loadingStatus = useAsyncStatuses([delegationStatus, claimRewards.loadingStatus]);
  const validatorDetailRoute = `/wallet/staking/${validatorAddress.chain.name}/${validatorAddress.id}`;
  return {
    delegation,
    claimRewards,
    loadingStatus,
    validatorDetailRoute
  };
}
const ValidatorDetailContext = createContext(null);
function useValidatorDetail() {
  const state = useContext(ValidatorDetailContext);
  if (!state)
    throw new Error("ValidatorDetailContext missing");
  return state;
}
function defaultRenderError$3(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function ValidatorDetailProvider({
  children,
  renderLoading,
  renderError = defaultRenderError$3,
  ...props
}) {
  const state = useValidatorDetailState(props);
  if (renderLoading && state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  if (renderLoading && !state.delegation)
    return renderLoading();
  return /* @__PURE__ */ React.createElement(ValidatorDetailContext.Provider, {
    value: state
  }, children);
}

function useGovernanceState() {
  const { onomyNomAsset: govAsset } = useOnomyApp();
  const [myAddress, myAddressStatus] = useAddress(govAsset);
  const [allProposals, allProposalsStatus] = useProposals();
  const [myProposals, myProposalsStatus] = useProposals({ depositor: myAddress == null ? void 0 : myAddress.id });
  const [myVotedProposals, myVotedProposalsStatus] = useProposals({ voter: myAddress == null ? void 0 : myAddress.id });
  const govAssetBalance = useAssetBalance(govAsset);
  const loadingStatus = useAsyncStatuses([
    myAddressStatus,
    allProposalsStatus,
    myProposalsStatus,
    myVotedProposalsStatus
  ]);
  const activeProposals = useMemo(() => {
    return allProposals.filter((proposal) => {
      return proposal.status === "PROPOSAL_STATUS_VOTING_PERIOD";
    });
  }, [allProposals]);
  const passedProposals = useMemo(() => {
    return allProposals.filter((proposal) => {
      return proposal.status === "PROPOSAL_STATUS_PASSED";
    });
  }, [allProposals]);
  const rejectedProposals = useMemo(() => {
    return allProposals.filter((proposal) => {
      return proposal.status === "PROPOSAL_STATUS_REJECTED" || proposal.status === "PROPOSAL_STATUS_FAILED";
    });
  }, [allProposals]);
  const failedProposals = useMemo(() => {
    return allProposals.filter((proposal) => {
      return proposal.status === "PROPOSAL_STATUS_FAILED";
    });
  }, [allProposals]);
  return {
    governanceRoute: "/governance",
    govAsset,
    govAssetBalance,
    myAddress,
    allProposals,
    myProposals,
    myVotedProposals,
    activeProposals,
    passedProposals,
    rejectedProposals,
    failedProposals,
    loadingStatus
  };
}
const GovernanceContext = createContext(null);
function useGovernance() {
  const state = useContext(GovernanceContext);
  if (!state)
    throw new Error("GovernanceContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN$6() {
  return /* @__PURE__ */ React.createElement(PlaceholderLoadingIndicator, {
    text: "Loading governance"
  });
}
function DEFAULT_ERROR_RENDER_FN$6(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function GovernanceProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN$6,
  renderError = DEFAULT_ERROR_RENDER_FN$6
}) {
  const state = useGovernanceState();
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(GovernanceContext.Provider, {
    value: state
  }, children);
}

function useProposalCreateState() {
  const form = useSubmitProposalForm();
  return form;
}
const ProposalCreateContext = createContext(null);
function useProposalCreate() {
  const state = useContext(ProposalCreateContext);
  if (!state)
    throw new Error("ProposalCreateContext missing");
  return state;
}
function ProposalCreateProvider({ children }) {
  const state = useProposalCreateState();
  return /* @__PURE__ */ React.createElement(ProposalCreateContext.Provider, {
    value: state
  }, children);
}

function useProposalDepositForm({
  autoSetNeededAmount: autoSetNeededAmountProp = false,
  ...props
}) {
  var _a, _b;
  const { hub } = useOnomyHub();
  const [attempt, txStatus, txReset] = useAsyncProcess();
  const [txDetails, setTxDetails] = useState(null);
  const [proposal, proposalStatus] = useProposal(props);
  const [{ asset: govAsset, feeAsset }, assetStatus] = useStakingAssets(props.chain);
  const { asset = govAsset } = proposal != null ? proposal : {};
  const [balance, balanceStatus] = useAssetBalance(asset);
  const [amountFixed, setAmountFixed] = useState("");
  const [autoSetNeededAmount, setAutoSetNeededAmount] = useState(autoSetNeededAmountProp);
  const toggleAutoSetNeededAmount = useCallback(() => setAutoSetNeededAmount((v) => !v), []);
  const [amount, amountStatus] = useDetailedBalance(asset, amountFixed);
  const [minDepositAmount, minDepositAmountStatus] = useProposalMinDeposit(props.chain);
  const depositNeededFixed = minDepositAmount.amount - ((_b = (_a = proposal == null ? void 0 : proposal.totalDeposits[0]) == null ? void 0 : _a.amount) != null ? _b : BigInt(0));
  const [depositNeeded, depositNeededStatus] = useDetailedBalance(asset, depositNeededFixed.toString(10));
  useEffect(() => {
    if (autoSetNeededAmount)
      setAmountFixed(depositNeededFixed.toString(10));
  }, [depositNeededFixed, autoSetNeededAmount]);
  const [amountInput, setAmountInput] = useFixedAmountInput({
    value: amountFixed,
    onChangeValue: setAmountFixed
  });
  const inputParams = useMemo(() => {
    return {
      proposalId: props.id,
      amountFixed: new BigNumber(amountFixed)
    };
  }, [props.id, amountFixed]);
  const gas = useGasForm(asset, feeAsset, async () => {
    if (!hub)
      return "0";
    const cosmosChain = OnomyHubUtils.getCosmosChain(hub.chains, props.chain);
    if (!cosmosChain)
      return "0";
    const res = await cosmosChain.client.feesForDepositToProposal(inputParams);
    const [denomAmount] = res.amount;
    return denomAmount.amount;
  }, [hub, inputParams]);
  const loadingStatus = useAsyncStatuses([
    proposalStatus,
    assetStatus,
    balanceStatus,
    amountStatus,
    minDepositAmountStatus,
    depositNeededStatus
  ]);
  const hasEnoughBalance = useMemo(() => {
    let total = amount.amount;
    if (amount.asset.id === gas.gasAmount.asset.id)
      total += gas.gasAmount.amount;
    return balance.amount >= total;
  }, [
    amount.amount,
    amount.asset.id,
    gas.gasAmount.asset.id,
    gas.gasAmount.amount,
    balance.amount
  ]);
  const formErrors = useMemo(() => {
    const errors = {};
    if (!inputParams.proposalId) {
      errors.proposalId = "Proposal not set";
    }
    if (!inputParams.amountFixed) {
      errors.amountInput = "Amount is required";
    } else {
      const amountFixedValue = new BigNumber(inputParams.amountFixed);
      if (amountFixedValue.lte(0) || amountFixedValue.isNaN()) {
        errors.amountInput = "Amount must be greater than 0";
      }
    }
    if (!gas.hasEnoughGasBalance)
      errors.gas = `Insuffcient ${gas.gasAsset.meta.symbol}`;
    if (!hasEnoughBalance)
      errors.amountInput = `Insuffcient ${balance.asset.meta.symbol}`;
    return Object.keys(errors).length > 0 ? errors : null;
  }, [
    inputParams.proposalId,
    inputParams.amountFixed,
    gas.hasEnoughGasBalance,
    gas.gasAsset.meta.symbol,
    hasEnoughBalance,
    balance.asset.meta.symbol
  ]);
  const canSubmit = useMemo(() => {
    return !formErrors && !loadingStatus.pending && !loadingStatus.error && !!gas.gasPrice;
  }, [formErrors, gas.gasPrice, loadingStatus]);
  const reset = useCallback(() => {
    setAmountFixed("");
    txReset();
    setTxDetails(null);
  }, [txReset]);
  const submit = useCallback(() => {
    return attempt(async () => {
      if (!hub || !canSubmit)
        return;
      const cosmosChain = OnomyHubUtils.getCosmosChain(hub.chains, props.chain);
      if (!cosmosChain)
        return;
      await cosmosChain.client.depositToProposal({
        ...inputParams,
        feeMultiplier: parseInt(gas.gasPrice, 10)
      });
      await hub.proposals.forceGet(props.chain.name, inputParams.proposalId);
      const balanceAfter = await hub.balances.forceGet(balance.asset);
      const result = {
        proposalId: inputParams.proposalId,
        chain: props.chain,
        amount,
        gasAmount: gas.gasAmount,
        balanceBefore: balance,
        balanceAfter
      };
      setTxDetails(result);
    });
  }, [
    attempt,
    hub,
    canSubmit,
    amount,
    balance,
    gas.gasAmount,
    gas.gasPrice,
    inputParams,
    props.chain
  ]);
  return {
    loadingStatus,
    txStatus,
    txDetails,
    proposal,
    asset,
    balance,
    amount,
    minDepositAmount,
    depositNeeded,
    amountInput,
    autoSetNeededAmount,
    gas,
    setAmountInput,
    setAutoSetNeededAmount,
    toggleAutoSetNeededAmount,
    hasEnoughBalance,
    formErrors,
    canSubmit,
    reset,
    submit
  };
}

const ProposalDepositContext = createContext(null);
function useProposalDeposit() {
  const state = useContext(ProposalDepositContext);
  if (!state)
    throw new Error("ProposalDepositContext missing");
  return state;
}
function defaultRenderLoading() {
  return /* @__PURE__ */ React.createElement(PlaceholderLoadingIndicator, {
    text: "Loading proposal"
  });
}
function defaultRenderError$2(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function ProposalDepositProvider({
  children,
  renderLoading = defaultRenderLoading,
  renderError = defaultRenderError$2,
  ...props
}) {
  const state = useProposalDepositForm(props);
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  if (!state.proposal)
    return renderLoading();
  return /* @__PURE__ */ React.createElement(ProposalDepositContext.Provider, {
    value: state
  }, children);
}

function useProposalDetailState(props) {
  var _a, _b;
  const [proposal, proposalStatus] = useProposal(props);
  const votesChartData = useMemo(() => {
    return proposal ? calculateVotesChartData(proposal) : [0, 0, 0];
  }, [proposal]);
  const isVotingPeriod = (proposal == null ? void 0 : proposal.status) === "PROPOSAL_STATUS_VOTING_PERIOD";
  const isDepositPeriod = (proposal == null ? void 0 : proposal.status) === "PROPOSAL_STATUS_DEPOSIT_PERIOD";
  const [minDepositAmount, minDepositAmountStatus] = useProposalMinDeposit(props.chain);
  const depositNeededFixed = minDepositAmount.amount - ((_b = (_a = proposal == null ? void 0 : proposal.totalDeposits[0]) == null ? void 0 : _a.amount) != null ? _b : BigInt(0));
  const [depositNeeded, depositNeededStatus] = useDetailedBalance(minDepositAmount.asset, depositNeededFixed.toString(10));
  const loadingStatus = useAsyncStatuses([
    proposalStatus,
    minDepositAmountStatus,
    depositNeededStatus
  ]);
  return {
    chain: props.chain,
    proposalId: props.id,
    proposal,
    votesChartData,
    isVotingPeriod,
    isDepositPeriod,
    minDepositAmount,
    depositNeeded,
    loadingStatus
  };
}
const ProposalDetailContext = createContext(null);
function useProposalDetail() {
  const state = useContext(ProposalDetailContext);
  if (!state)
    throw new Error("ProposalDetailContext missing");
  return state;
}
function defaultRenderError$1(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function ProposalDetailProvider({
  children,
  renderLoading,
  renderError = defaultRenderError$1,
  ...props
}) {
  const state = useProposalDetailState(props);
  if (renderLoading && state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  if (!state.proposal && renderLoading)
    return renderLoading();
  return /* @__PURE__ */ React.createElement(ProposalDetailContext.Provider, {
    value: state
  }, children);
}

const ProposalVoteContext = createContext(null);
function useProposalVote() {
  const state = useContext(ProposalVoteContext);
  if (!state)
    throw new Error("Missing ProposalVoteContext");
  return state;
}
function defaultRenderError(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function ProposalVoteProvider({
  children,
  renderLoading,
  renderError = defaultRenderError,
  ...props
}) {
  const state = useGovVoteForm(props);
  if (renderLoading && (state.loadingStatus.pending || !state.proposal))
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(ProposalVoteContext.Provider, {
    value: state
  }, children);
}

const TITLE_MAX_LENGTH$1 = 140;
const DESCRIPTION_MAX_LENGTH$1 = 2500;
function useFundAccountProposalContent({
  chain,
  type = OnomyProposals.tree.onomyprotocol.dao.v1.FundAccountProposal
}) {
  const { hub } = useOnomyHub();
  const [{ asset }, assetStatus] = useStakingAssets(chain);
  const [title, setTitle] = useState("");
  const [description, setDescription] = useState("");
  const [recipient, setRecipient] = useState("");
  const [amountFixed, setAmountFixed] = useState("");
  const [amount, amountStatus] = useDetailedBalance(asset, amountFixed);
  const inputParams = useMemo(() => {
    return {
      title,
      description,
      recipient,
      amount: [
        {
          denom: asset.id,
          amount: amountFixed
        }
      ]
    };
  }, [title, description, recipient, asset.id, amountFixed]);
  const [amountInput, setAmountInput] = useFixedAmountInput({
    value: amountFixed,
    onChangeValue: setAmountFixed,
    meta: amount.asset.meta
  });
  const loadingStatus = useAsyncStatuses([assetStatus]);
  const formErrors = useMemo(() => {
    const params = inputParams;
    const errors = {};
    if (!params.title) {
      errors.title = "Title is required";
    } else if (params.title.length >= TITLE_MAX_LENGTH$1) {
      errors.title = `Title must be less than ${TITLE_MAX_LENGTH$1} characters`;
    }
    if (!params.description) {
      errors.description = "Description is required";
    } else if (params.description.length >= DESCRIPTION_MAX_LENGTH$1) {
      errors.description = `Description must be less than ${DESCRIPTION_MAX_LENGTH$1} characters`;
    }
    const cosmosChain = hub ? OnomyHubUtils.getCosmosChain(hub.chains, chain) : null;
    if (!params.recipient || !cosmosChain) {
      errors.recipient = "Recipient is required";
    } else if (!cosmosChain.isAddressValid({
      chain,
      id: params.recipient
    })) {
      errors.recipient = "Recipient must be a valid address";
    }
    if (!params.amount[0].amount) {
      errors.amountInput = "Amount is required";
    } else {
      const amountFixedValue = new BigNumber(params.amount[0].amount);
      if (amountFixedValue.lte(0) || amountFixedValue.isNaN()) {
        errors.amountInput = "Amount must be greater than 0";
      }
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [chain, inputParams, hub]);
  const canSubmit = useMemo(() => {
    return !formErrors;
  }, [formErrors]);
  const reset = useCallback(() => {
    setTitle("");
    setDescription("");
    setRecipient("");
    setAmountFixed("");
  }, []);
  const encoded = useMemo(() => {
    if (!canSubmit)
      return null;
    try {
      return type.encode(inputParams);
    } catch (e) {
      console.warn(`Error encoding ${type.typeUrl}`, e);
      return null;
    }
  }, [type, canSubmit, inputParams]);
  return {
    TITLE_MAX_LENGTH: TITLE_MAX_LENGTH$1,
    DESCRIPTION_MAX_LENGTH: DESCRIPTION_MAX_LENGTH$1,
    loadingStatus,
    assetStatus,
    amountStatus,
    chain,
    type,
    amount,
    title,
    description,
    amountInput,
    recipient,
    formErrors,
    setTitle,
    setDescription,
    setAmountInput,
    setRecipient,
    canSubmit,
    encoded,
    reset
  };
}

const BLANK_JSON$1 = "{}";
function useUnknownProposalContent({
  chain,
  type
}) {
  const [json, setJson] = useState(BLANK_JSON$1);
  const parsed = useMemo(() => {
    try {
      return JSON.parse(json);
    } catch (e) {
      return null;
    }
  }, [json]);
  const encoded = useMemo(() => {
    if (!parsed)
      return null;
    try {
      return type.encode(parsed);
    } catch (e) {
      return null;
    }
  }, [parsed, type]);
  const formErrors = useMemo(() => {
    const errors = {};
    if (!encoded)
      errors.json = "Invalid proposal content";
    if (!parsed)
      errors.json = "Invalid JSON";
    return Object.keys(errors).length > 0 ? errors : null;
  }, [parsed, encoded]);
  const canSubmit = !formErrors;
  const reset = useCallback(() => {
    setJson(BLANK_JSON$1);
  }, []);
  return {
    json,
    parsed,
    chain,
    type,
    setJson,
    encoded,
    formErrors,
    canSubmit,
    reset
  };
}

const TITLE_MAX_LENGTH = 140;
const DESCRIPTION_MAX_LENGTH = 2500;
const BLANK_JSON = `{
  "name": "Cosmos Hub Atom",
  "symbol": "ATOM",
  "description": "The native staking token of the Cosmos Hub.",
  "denomUnits": [
    {
      "denom": "uatom",
      "exponent": 0,
      "aliases": [
        "microatom"
      ]
    },
    {
      "denom": "matom",
      "exponent": 3,
      "aliases": [
        "milliatom"
      ]
    },
    {
      "denom": "atom",
      "exponent": 6,
      "aliases": []
    }
  ],
  "base": "uatom",
  "display": "atom"
}`;
function useDenomMetadataProposalContent({
  chain,
  type = OnexProposals.tree.pendulumlabs.market.market.DenomMetadataProposal
}) {
  const [address, addressStatus] = useAddress(chain);
  const [title, setTitle] = useState("");
  const [description, setDescription] = useState("");
  const [metadataJson, setMetadataJson] = useState(BLANK_JSON);
  const parsed = useMemo(() => {
    try {
      return JSON.parse(metadataJson);
    } catch (e) {
      return null;
    }
  }, [metadataJson]);
  const metadata = useMemo(() => {
    if (!parsed)
      return null;
    try {
      return Metadata.toProtoMsg(parsed);
    } catch (e) {
      return null;
    }
  }, [parsed]);
  const inputParams = useMemo(() => {
    var _a;
    return {
      sender: (_a = address == null ? void 0 : address.id) != null ? _a : "",
      title,
      description,
      metadata: metadata ? parsed : null
    };
  }, [title, description, metadata, parsed, address == null ? void 0 : address.id]);
  const loadingStatus = useAsyncStatuses([addressStatus]);
  const formErrors = useMemo(() => {
    const params = inputParams;
    const errors = {};
    if (!params.title) {
      errors.title = "Title is required";
    } else if (params.title.length >= TITLE_MAX_LENGTH) {
      errors.title = `Title must be less than ${TITLE_MAX_LENGTH} characters`;
    }
    if (!params.description) {
      errors.description = "Description is required";
    } else if (params.description.length >= DESCRIPTION_MAX_LENGTH) {
      errors.description = `Description must be less than ${DESCRIPTION_MAX_LENGTH} characters`;
    }
    if (parsed && !inputParams.metadata) {
      errors.metadataJson = "Invalid format for metadata json";
    } else if (!parsed) {
      errors.metadataJson = "Metadata is not valid JSON";
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [inputParams, parsed]);
  const canSubmit = useMemo(() => {
    return !formErrors;
  }, [formErrors]);
  const reset = useCallback(() => {
    setTitle("");
    setDescription("");
    setMetadataJson(BLANK_JSON);
  }, []);
  const encoded = useMemo(() => {
    if (!canSubmit || !inputParams.metadata)
      return null;
    try {
      return type.encode(inputParams);
    } catch (e) {
      console.warn(`Error encoding ${type.typeUrl}`, e);
      return null;
    }
  }, [type, canSubmit, inputParams]);
  return {
    TITLE_MAX_LENGTH,
    DESCRIPTION_MAX_LENGTH,
    loadingStatus,
    addressStatus,
    chain,
    type,
    title,
    description,
    metadataJson,
    metadata: parsed,
    formErrors,
    setTitle,
    setDescription,
    setMetadataJson,
    canSubmit,
    encoded,
    reset
  };
}

function useCancelOrderForm({ orderId = "" } = {}) {
  const onex = useOnexClient();
  const [attemptTx, txStatus, resetTxStatus] = useAsyncProcess();
  const [txResult, setTxResult] = useState(null);
  const [uid, setUid] = useState(orderId);
  const doCancel = useCallback((id) => {
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a;
      return setTxResult((_a = await (onex == null ? void 0 : onex.cancelOrder({ uid: id }))) != null ? _a : null);
    });
  }, [onex, resetTxStatus, attemptTx]);
  const submitTx = useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    return doCancel(uid);
  }, [doCancel, uid]);
  const reset = useCallback(() => {
    resetTxStatus();
    setTxResult(null);
  }, [resetTxStatus]);
  useEffect(() => {
    setUid(orderId);
    reset();
  }, [orderId, reset]);
  return {
    txStatus,
    txResult,
    doCancel,
    submitTx,
    reset
  };
}

function useExchangeCancelOrderState(props) {
  const form = useCancelOrderForm(props);
  return form;
}
const ExchangeCancelOrderContext = createContext(null);
function useExchangeCancelOrder() {
  const state = useContext(ExchangeCancelOrderContext);
  if (!state)
    throw new Error("ExchangeCancelOrderContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN$5() {
  return /* @__PURE__ */ React.createElement(PlaceholderLoadingIndicator, {
    text: "Loading add drops form"
  });
}
function DEFAULT_ERROR_RENDER_FN$5(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function ExchangeCancelOrderProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN$5,
  renderError = DEFAULT_ERROR_RENDER_FN$5,
  ...props
}) {
  const state = useExchangeCancelOrderState(props);
  return /* @__PURE__ */ React.createElement(ExchangeCancelOrderContext.Provider, {
    value: state
  }, children);
}

const BN_ZERO = new BigNumber(0);
const BN_ONE = new BigNumber(1);
const DEFAULT_STATE = {
  rate: ["0", "0"],
  price: "0",
  inversePrice: "0",
  amountLastChanged: "base",
  amountBaseFixed: "0",
  amountQuoteFixed: "0"
};
const MAX_RATE_DENOMINATOR = "18446744073709551614";
function calculateRate({
  price,
  baseAsset,
  quoteAsset
}) {
  let bnPrice = new BigNumber(price);
  if (bnPrice.isNaN())
    bnPrice = BN_ZERO;
  const expShift = baseAsset.meta.decimalPlaces - quoteAsset.meta.decimalPlaces;
  let bnInversePrice = BN_ONE.dividedBy(bnPrice).shiftedBy(-1 * expShift);
  if (!bnInversePrice.isFinite())
    bnInversePrice = BN_ZERO;
  bnPrice = bnPrice.shiftedBy(expShift);
  const rate = bnPrice.eq(0) ? ["0", "0"] : bnPrice.toFraction(MAX_RATE_DENOMINATOR).map((x) => x.toFixed(0));
  while (BigNumber(rate[0]).gte(MAX_RATE_DENOMINATOR)) {
    rate[0] = rate[0].slice(0, rate[0].length - 1);
    rate[1] = rate[1].slice(0, rate[1].length - 1);
  }
  return {
    price,
    bnPrice,
    bnInversePrice,
    rate,
    inversePrice: bnInversePrice.toString(10)
  };
}
function calculatePrice({
  baseAmountFixed,
  quoteAmountFixed,
  baseAsset,
  quoteAsset
}) {
  var _a, _b;
  const bnQuote = new BigNumber(quoteAmountFixed).shiftedBy(-1 * ((_a = quoteAsset.meta.decimalPlaces) != null ? _a : 18));
  const bnBase = new BigNumber(baseAmountFixed).shiftedBy(-1 * ((_b = baseAsset.meta.decimalPlaces) != null ? _b : 18));
  const price = bnQuote.dividedBy(bnBase).toString(10);
  return calculateRate({ price, baseAsset, quoteAsset });
}
function usePriceInputs({
  baseAsset,
  quoteAsset,
  updateOtherAsset = true
}) {
  const [{ rate, price, inversePrice, amountBaseFixed, amountQuoteFixed }, setState] = useState(DEFAULT_STATE);
  const priceFixed = useFixedAmountFromDisplay(price, quoteAsset);
  const [pricePerBase, pricePerBaseStatus] = useDetailedBalance(quoteAsset, priceFixed);
  const inversePriceFixed = useFixedAmountFromDisplay(inversePrice, baseAsset);
  const [inversePricePerBase, inversePricePerBaseStatus] = useDetailedBalance(baseAsset, inversePriceFixed);
  const setPrice = useCallback((newPrice) => {
    setState((current) => {
      const {
        bnPrice: calculatedBnPrice,
        inversePrice: calculatedInversePrice,
        rate: calculatedRate
      } = calculateRate({
        price: newPrice,
        baseAsset,
        quoteAsset
      });
      if (current.amountLastChanged === "base") {
        const bnAmount2 = new BigNumber(current.amountBaseFixed);
        const calculatedQuote2 = bnAmount2.multipliedBy(calculatedBnPrice);
        return {
          ...current,
          amountQuoteFixed: calculatedQuote2.isNaN() || !calculatedQuote2.isFinite() ? "0" : calculatedQuote2.toFixed(0),
          rate: calculatedRate,
          price: newPrice,
          inversePrice: calculatedInversePrice
        };
      }
      const bnAmount = new BigNumber(current.amountQuoteFixed);
      const calculatedQuote = bnAmount.dividedBy(calculatedBnPrice);
      return {
        ...current,
        amountBaseFixed: calculatedQuote.isNaN() || !calculatedQuote.isFinite() ? "0" : calculatedQuote.toFixed(0),
        rate: calculatedRate,
        price: newPrice,
        inversePrice: calculatedInversePrice
      };
    });
  }, [baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]);
  const setAmountBaseFixed = useCallback((newAmountFixed) => {
    setState((current) => {
      if (updateOtherAsset) {
        const bnAmount = new BigNumber(newAmountFixed);
        const expShift = baseAsset.meta.decimalPlaces - quoteAsset.meta.decimalPlaces;
        const bnPrice = new BigNumber(current.price).shiftedBy(expShift);
        const calculatedQuote = bnAmount.multipliedBy(bnPrice);
        return {
          ...current,
          amountLastChanged: "base",
          amountBaseFixed: newAmountFixed,
          amountQuoteFixed: calculatedQuote.isNaN() || !calculatedQuote.isFinite() ? "0" : calculatedQuote.toFixed(0)
        };
      }
      const {
        price: calculatedPrice,
        inversePrice: calculatedInversePrice,
        rate: calculatedRate
      } = calculatePrice({
        baseAsset,
        quoteAsset,
        baseAmountFixed: newAmountFixed,
        quoteAmountFixed: current.amountQuoteFixed
      });
      return {
        ...current,
        price: calculatedPrice,
        inversePrice: calculatedInversePrice,
        rate: calculatedRate,
        amountLastChanged: "base",
        amountBaseFixed: newAmountFixed
      };
    });
  }, [updateOtherAsset, baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]);
  const setAmountQuoteFixed = useCallback((newAmountFixed) => {
    setState((current) => {
      if (updateOtherAsset) {
        const bnAmount = new BigNumber(newAmountFixed);
        const expShift = baseAsset.meta.decimalPlaces - quoteAsset.meta.decimalPlaces;
        const bnPrice = new BigNumber(current.price).shiftedBy(expShift);
        const calculatedBase = bnAmount.dividedBy(bnPrice);
        return {
          ...current,
          amountLastChanged: "quote",
          amountQuoteFixed: newAmountFixed,
          amountBaseFixed: calculatedBase.isNaN() || !calculatedBase.isFinite() ? "0" : calculatedBase.toFixed(0)
        };
      }
      const {
        price: calculatedPrice,
        inversePrice: calculatedInversePrice,
        rate: calculatedRate
      } = calculatePrice({
        baseAsset,
        quoteAsset,
        quoteAmountFixed: newAmountFixed,
        baseAmountFixed: current.amountBaseFixed
      });
      return {
        ...current,
        price: calculatedPrice,
        inversePrice: calculatedInversePrice,
        rate: calculatedRate,
        amountLastChanged: "quote",
        amountQuoteFixed: newAmountFixed
      };
    });
  }, [updateOtherAsset, baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]);
  const reset = useCallback(() => setState(DEFAULT_STATE), []);
  return {
    rate,
    price,
    inversePrice,
    amountBaseFixed,
    amountQuoteFixed,
    priceFixed,
    pricePerBase,
    pricePerBaseStatus,
    inversePriceFixed,
    inversePricePerBase,
    inversePricePerBaseStatus,
    setPrice,
    setAmountBaseFixed,
    setAmountQuoteFixed,
    reset
  };
}

function useCreateOrderForm({
  baseDenom = "anom",
  quoteDenom = "usdt",
  orderDirection = "buy",
  orderType: orderTypeProp = "limit"
}) {
  var _a;
  const onex = useOnexClient();
  const chain = (_a = onex == null ? void 0 : onex.getBlockchain()) != null ? _a : null;
  const [direction, setDirection] = useState(orderDirection);
  const [baseAsset, baseAssetStatus] = useAsset$1(baseDenom, chain);
  const [quoteAsset, quoteAssetStatus] = useAsset$1(quoteDenom, chain);
  const assetStatus = useAsyncStatuses([baseAssetStatus, quoteAssetStatus]);
  const {
    rate,
    price,
    amountBaseFixed,
    amountQuoteFixed,
    pricePerBase,
    pricePerBaseStatus,
    setPrice,
    setAmountBaseFixed,
    setAmountQuoteFixed,
    reset: resetPriceInputs
  } = usePriceInputs({ baseAsset, quoteAsset });
  const [stopPrice, setStopPrice] = useState("0");
  const stopPriceFixed = useFixedAmountFromDisplay(stopPrice, quoteAsset);
  const [stopPricePerBase, stopPricePerBaseStatus] = useDetailedBalance(quoteAsset, stopPriceFixed);
  const [currentPricePerBase, currentPricePerBaseStatus] = useOnexDetailedPoolPrice({
    baseAsset,
    quoteAsset
  });
  const [baseBalance, baseBalanceStatus] = useAssetBalance(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = useAssetBalance(quoteAsset);
  const balanceStatus = useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);
  const [orderType, setOrderType] = useState(orderTypeProp);
  const [txResult, setTxResult] = useState(null);
  const [attemptTx, txStatus, resetTxStatus] = useAsyncProcess();
  const [amountQuote, amountQuoteStatus] = useDetailedBalance(quoteAsset, amountQuoteFixed);
  const [amountBase, amountBaseStatus] = useDetailedBalance(baseAsset, amountBaseFixed);
  const amountStatus = useAsyncStatuses([amountQuoteStatus, amountBaseStatus]);
  const [bookSell, bookSellStatus] = useOnexOrderBook({ baseAsset, quoteAsset, direction: "sell" });
  const [bookBuy, bookBuyStatus] = useOnexOrderBook({ baseAsset, quoteAsset, direction: "buy" });
  const bookStatus = useAsyncStatuses([bookSellStatus, bookBuyStatus]);
  const bidAsset = direction === "buy" ? quoteAsset : baseAsset;
  const askAsset = direction === "buy" ? baseAsset : quoteAsset;
  const bidBalance = direction === "buy" ? quoteBalance : baseBalance;
  const askBalance = direction === "buy" ? baseBalance : quoteBalance;
  const bidAmountFixed = direction === "buy" ? amountQuoteFixed : amountBaseFixed;
  const askAmountFixed = direction === "buy" ? amountBaseFixed : amountQuoteFixed;
  const bidAmount = direction === "buy" ? amountQuote : amountBase;
  const askAmount = direction === "buy" ? amountBase : amountQuote;
  const setBidAmountFixed = direction === "buy" ? setAmountQuoteFixed : setAmountBaseFixed;
  const setAskAmountFixed = direction === "buy" ? setAmountBaseFixed : setAmountQuoteFixed;
  const [bidAmountInput, setBidAmountInput] = useFixedAmountInput({
    value: bidAmountFixed,
    onChangeValue: setBidAmountFixed,
    meta: bidAsset.meta
  });
  const [askAmountInput, setAskAmountInput] = useFixedAmountInput({
    value: askAmountFixed,
    onChangeValue: setAskAmountFixed,
    meta: askAsset.meta
  });
  const inputParams = useMemo(() => {
    if (direction === "buy") {
      return {
        denomAsk: baseDenom,
        denomBid: quoteDenom,
        orderType,
        amount: amountQuoteFixed,
        rate: [rate[1], rate[0]]
      };
    }
    return {
      denomAsk: quoteDenom,
      denomBid: baseDenom,
      orderType,
      amount: amountBaseFixed,
      rate: [rate[0], rate[1]]
    };
  }, [direction, baseDenom, quoteDenom, orderType, amountQuoteFixed, amountBaseFixed, rate]);
  const validationErrors = useMemo(() => {
    const errors = {};
    const bnPrice = new BigNumber(price);
    const bnAmountQuote = new BigNumber(amountQuoteFixed);
    const bnAmountBase = new BigNumber(amountBaseFixed);
    if (bnPrice.isNaN() || bnPrice.lte(0)) {
      errors.price = "Must be a number > 0";
    }
    if (bnAmountQuote.isNaN() || bnAmountQuote.lte(0)) {
      errors.amountQuote = "Must be a number > 0";
    }
    if (bnAmountBase.isNaN() || bnAmountBase.lte(0)) {
      errors.amountBase = "Must be a number > 0";
    }
    if (direction === "buy" && bnAmountQuote.gt(quoteBalance.amount.toString())) {
      errors.amountQuote = "Exceeds available balance";
    }
    if (direction === "sell" && bnAmountBase.gt(baseBalance.amount.toString())) {
      errors.amountBase = "Exceeds available balance";
    }
    if (direction === "buy") {
      if (errors.amountBase)
        errors.amountAsk = errors.amountBase;
      if (errors.amountQuote)
        errors.amountBid = errors.amountQuote;
    } else {
      if (errors.amountBase)
        errors.amountBid = errors.amountBase;
      if (errors.amountQuote)
        errors.amountAsk = errors.amountQuote;
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [
    direction,
    price,
    amountQuoteFixed,
    amountBaseFixed,
    quoteBalance.amount,
    baseBalance.amount
  ]);
  const loadingStatus = useAsyncStatuses([assetStatus, balanceStatus, currentPricePerBaseStatus]);
  const canSubmit = !validationErrors && loadingStatus.finished;
  const submitTx = useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    if (!canSubmit)
      return Promise.resolve();
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a2;
      return setTxResult((_a2 = await (onex == null ? void 0 : onex.createOrder(inputParams))) != null ? _a2 : null);
    });
  }, [canSubmit, inputParams, resetTxStatus, attemptTx, onex]);
  const reset = useCallback(() => {
    resetPriceInputs();
    setOrderType("limit");
    resetTxStatus();
    setTxResult(null);
  }, [resetPriceInputs, resetTxStatus]);
  useEffect(reset, [baseDenom, quoteDenom, orderDirection, orderType, reset]);
  return {
    txStatus,
    txResult,
    assetStatus,
    balanceStatus,
    loadingStatus,
    amountStatus,
    pricePerBaseStatus,
    stopPricePerBaseStatus,
    currentPricePerBaseStatus,
    bookBuyStatus,
    bookSellStatus,
    bookStatus,
    validationErrors,
    canSubmit,
    baseAsset,
    baseBalance,
    quoteAsset,
    quoteBalance,
    amountQuoteFixed,
    amountBaseFixed,
    baseDenom,
    quoteDenom,
    direction,
    price,
    stopPrice,
    orderType,
    pricePerBase,
    stopPricePerBase,
    currentPricePerBase,
    amountQuote,
    amountBase,
    bookSell,
    bookBuy,
    bidAsset,
    askAsset,
    bidBalance,
    askBalance,
    bidAmount,
    askAmount,
    bidAmountInput,
    askAmountInput,
    setBidAmountInput,
    setAskAmountInput,
    setDirection,
    setPrice,
    setOrderType,
    setAmountQuoteFixed,
    setAmountBaseFixed,
    submitTx,
    reset
  };
}

function useMarketOrderForm({
  baseDenom = "anom",
  quoteDenom = "usdt",
  orderDirection = "buy"
}) {
  var _a;
  const onex = useOnexClient();
  const chain = (_a = onex == null ? void 0 : onex.getBlockchain()) != null ? _a : null;
  const [direction, setDirection] = useState(orderDirection);
  const [baseAsset, baseAssetStatus] = useAsset$1(baseDenom, chain);
  const [quoteAsset, quoteAssetStatus] = useAsset$1(quoteDenom, chain);
  const assetStatus = useAsyncStatuses([baseAssetStatus, quoteAssetStatus]);
  const [baseBalance, baseBalanceStatus] = useAssetBalance(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = useAssetBalance(quoteAsset);
  const balanceStatus = useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);
  const [txResult, setTxResult] = useState(null);
  const [attemptTx, txStatus, resetTxStatus] = useAsyncProcess();
  const [hasAttemptedSubmit, setHasAttemptedSubmit] = useState(false);
  const [bidAmountFixed, setBidAmountFixed] = useState("0");
  const [slippagePercentage, setSlippagePercentage] = useState("3");
  const [estimatedAskAmount, estimateStatus] = useOnexMarketEstimate({
    denomAsk: direction === "buy" ? baseDenom : quoteDenom,
    denomBid: direction === "buy" ? quoteDenom : baseDenom,
    amountBid: bidAmountFixed
  });
  const bidAsset = direction === "buy" ? quoteAsset : baseAsset;
  const askAsset = direction === "buy" ? baseAsset : quoteAsset;
  const bidBalance = direction === "buy" ? quoteBalance : baseBalance;
  const askBalance = direction === "buy" ? baseBalance : quoteBalance;
  const [bidAmount, bidAmountStatus] = useDetailedBalance(bidAsset, bidAmountFixed);
  const [askAmount, askAmountStatus] = useDetailedBalance(askAsset, estimatedAskAmount);
  const amountStatus = useAsyncStatuses([estimateStatus, bidAmountStatus, askAmountStatus]);
  const baseAmount = direction === "buy" ? askAmount : bidAmount;
  const quoteAmount = direction === "buy" ? bidAmount : askAmount;
  const [currentPrice, currentPriceStatus] = useOnexPoolPrice({ baseDenom, quoteDenom });
  const currentPriceFixed = useFixedAmountFromDisplay(currentPrice, quoteAsset);
  const [currentPricePerBase, currentPricePerBaseStatus] = useDetailedBalance(quoteAsset, currentPriceFixed);
  const [bidAmountInput, setBidAmountInput] = useFixedAmountInput({
    value: bidAmountFixed,
    onChangeValue: setBidAmountFixed,
    meta: bidAsset.meta
  });
  const estimatedPricePerBaseFixed = useMemo(() => {
    const bnEstAskFixed = new BigNumber(estimatedAskAmount);
    const bnBidAmountFixed = new BigNumber(bidAmountFixed);
    if (bnEstAskFixed.isNaN() || bnEstAskFixed.eq(0) || bnBidAmountFixed.isNaN() || bnBidAmountFixed.eq(0)) {
      return "0";
    }
    if (direction === "buy") {
      const expShift2 = bidAsset.meta.decimalPlaces - askAsset.meta.decimalPlaces;
      const bnPrice2 = bnBidAmountFixed.dividedBy(bnEstAskFixed).shiftedBy(expShift2).shiftedBy(bidAsset.meta.decimalPlaces);
      return bnPrice2.toFixed(0);
    }
    const expShift = askAsset.meta.decimalPlaces - bidAsset.meta.decimalPlaces;
    const bnPrice = bnEstAskFixed.dividedBy(bnBidAmountFixed).shiftedBy(expShift).shiftedBy(askAsset.meta.decimalPlaces);
    return bnPrice.toFixed(0);
  }, [
    estimatedAskAmount,
    bidAmountFixed,
    bidAsset.meta.decimalPlaces,
    askAsset.meta.decimalPlaces,
    direction
  ]);
  const [estimatedPricePerBase, estimatedPricePerBaseStatus] = useDetailedBalance(quoteAsset, estimatedPricePerBaseFixed);
  const inputParams = useMemo(() => {
    const slippage = new BigNumber(slippagePercentage).times(100).toFixed(0);
    if (direction === "buy") {
      return {
        denomAsk: baseDenom,
        denomBid: quoteDenom,
        amountBid: bidAmountFixed,
        amountAsk: estimatedAskAmount,
        slippage
      };
    }
    return {
      denomAsk: quoteDenom,
      denomBid: baseDenom,
      amountBid: bidAmountFixed,
      amountAsk: estimatedAskAmount,
      slippage
    };
  }, [slippagePercentage, direction, baseDenom, quoteDenom, bidAmountFixed, estimatedAskAmount]);
  const validationErrors = useMemo(() => {
    const errors = {};
    const bnAmountBid = new BigNumber(inputParams.amountBid);
    const bnAmountAsk = new BigNumber(inputParams.amountAsk);
    const bnSlippage = new BigNumber(inputParams.slippage);
    if (bnAmountBid.isNaN() || !bnAmountBid.isFinite() || bnAmountBid.lte(0)) {
      errors.amountBid = "Must be a number > 0";
    }
    if (bnAmountAsk.isNaN() || !bnAmountAsk.isFinite() || bnAmountAsk.lte(0)) {
      errors.amountAsk = "Must be a number > 0";
    }
    if (bnSlippage.isNaN() || !bnSlippage.isFinite() || bnSlippage.lte(0)) {
      errors.slippage = "Must be a number > 0";
    }
    if (bnSlippage.gte(1e4)) {
      errors.slippage = "Must be a number < 100";
    }
    if (estimateStatus.error) {
      errors.liquidity = "Insufficient liquidity in pool";
    }
    if (bnAmountBid.gt(bidBalance.amount.toString())) {
      errors.amountBid = "Exceeds available balance";
    }
    if (bnAmountAsk.gt(askBalance.amount.toString())) {
      errors.amountAsk = "Exceeds available balance";
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [inputParams, estimateStatus.error, bidBalance.amount, askBalance.amount]);
  const loadingStatus = useAsyncStatuses([assetStatus, balanceStatus]);
  const canSubmit = !validationErrors && loadingStatus.finished && estimateStatus.finished;
  const submitTx = useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    setHasAttemptedSubmit(true);
    if (!canSubmit)
      return Promise.resolve();
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a2;
      return setTxResult((_a2 = await (onex == null ? void 0 : onex.marketOrder(inputParams))) != null ? _a2 : null);
    });
  }, [canSubmit, inputParams, resetTxStatus, attemptTx, onex]);
  const reset = useCallback(() => {
    setHasAttemptedSubmit(false);
    resetTxStatus();
    setBidAmountFixed("0");
    setTxResult(null);
  }, [resetTxStatus]);
  useEffect(reset, [baseDenom, quoteDenom, direction, reset]);
  useEffect(() => {
    setDirection(orderDirection);
  }, [orderDirection]);
  return {
    txStatus,
    txResult,
    assetStatus,
    balanceStatus,
    loadingStatus,
    estimateStatus,
    bidAmountStatus,
    askAmountStatus,
    amountStatus,
    currentPriceStatus,
    currentPricePerBaseStatus,
    estimatedPricePerAskStatus: estimatedPricePerBaseStatus,
    validationErrors,
    canSubmit,
    hasAttemptedSubmit,
    baseAsset,
    baseBalance,
    quoteAsset,
    quoteBalance,
    bidAsset,
    askAsset,
    bidAmount,
    askAmount,
    bidBalance,
    askBalance,
    baseAmount,
    quoteAmount,
    currentPricePerBase,
    estimatedPricePerBase,
    baseDenom,
    quoteDenom,
    direction,
    slippagePercentage,
    bidAmountInput,
    setDirection,
    setBidAmountInput,
    setSlippagePercentage,
    submitTx,
    reset
  };
}

function useExchangePairState({
  quoteAsset,
  baseAsset
}) {
  const createOrderForm = useCreateOrderForm({
    quoteDenom: quoteAsset.id,
    baseDenom: baseAsset.id
  });
  const marketOrderForm = useMarketOrderForm({
    quoteDenom: quoteAsset.id,
    baseDenom: baseAsset.id,
    orderDirection: createOrderForm.direction
  });
  const [dayTradeData, dayTradeDataStatus] = useOnexDayTradeFrame({ baseAsset, quoteAsset });
  const [history, historyStatus] = useOnexOrderHistory({ baseAsset, quoteAsset });
  const [pool, poolStatus] = useOnexPoolDetail({ baseAsset, quoteAsset });
  const [orders, ordersStatus] = useOnexMyActiveOrders({ quoteAsset, baseAsset });
  const {
    binOptions: bookBinOptions,
    bookBuy,
    bookSell,
    selectedBin: bookSelectedBin,
    setSelectedBin: setBookSelectedBin
  } = useIndexedOrderBookForm({
    baseAsset: createOrderForm.baseAsset,
    quoteAsset: createOrderForm.quoteAsset
  });
  const loadingStatus = useAsyncStatuses([
    createOrderForm.loadingStatus,
    dayTradeDataStatus,
    dayTradeData.loadingStatus,
    historyStatus,
    poolStatus,
    ordersStatus
  ]);
  return {
    loadingStatus,
    dayTradeDataStatus,
    quoteAsset,
    baseAsset,
    currentPricePerBase: createOrderForm.currentPricePerBase,
    lastPricePerBase: dayTradeData.lastPricePerBase,
    dayTradeData,
    createOrderForm,
    marketOrderForm,
    history,
    pool,
    orders,
    bookBuy,
    bookSell,
    bookBinOptions,
    bookSelectedBin,
    orderDirection: createOrderForm.direction,
    setOrderDirection: createOrderForm.setDirection,
    setBookSelectedBin
  };
}
const ExchangePairContext = createContext(null);
function useExchangePair() {
  const state = useContext(ExchangePairContext);
  if (!state)
    throw new Error("ExchangePairContext missing");
  return state;
}
const DEFAULT_LOADING_RENDER_FN$4 = () => /* @__PURE__ */ React.createElement(PlaceholderLoadingIndicator, {
  text: "Loading exchange pair"
});
const DEFAULT_ERROR_RENDER_FN$4 = (error) => /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
  error
});
function ExchangePairProviderInner({
  children,
  quoteAsset,
  baseAsset,
  renderLoading = DEFAULT_LOADING_RENDER_FN$4,
  renderError = DEFAULT_ERROR_RENDER_FN$4
}) {
  const state = useExchangePairState({ baseAsset, quoteAsset });
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(ExchangePairContext.Provider, {
    value: state
  }, children);
}
function ExchangePairProvider({
  children,
  getBaseSymbol,
  getQuoteSymbol,
  renderLoading = DEFAULT_LOADING_RENDER_FN$4,
  renderError = DEFAULT_ERROR_RENDER_FN$4
}) {
  const baseSymbol = getBaseSymbol();
  const quoteSymbol = getQuoteSymbol();
  const [allAssets, assetStatus] = useSupportedAssetList();
  const onexAssets = useMemo(() => allAssets.filter((x) => x.chain.name === OnomyHubConstants.ONEX_CHAIN_ID), [allAssets]);
  const baseAsset = useMemo(() => onexAssets.find((x) => x.meta.symbol.toLowerCase() === baseSymbol), [onexAssets, baseSymbol]);
  const quoteAsset = useMemo(() => onexAssets.find((x) => x.meta.symbol.toLowerCase() === quoteSymbol), [onexAssets, quoteSymbol]);
  if (assetStatus.pending)
    return renderLoading();
  if (assetStatus.error)
    return renderError(assetStatus.error);
  if (!baseAsset || !quoteAsset)
    return /* @__PURE__ */ React.createElement("h2", null, "Pair not found");
  return /* @__PURE__ */ React.createElement(ExchangePairProviderInner, {
    baseAsset,
    quoteAsset,
    renderError,
    renderLoading
  }, children);
}

const DEFAULT_AMOUNT_INPUTS_STATE = {
  amountBaseInputFixed: "0",
  amountQuoteInputFixed: "0",
  amountLastChanged: "base"
};
function useAddDropsForm({
  baseAsset,
  quoteAsset,
  pool
}) {
  const onex = useOnexClient();
  const [attemptTx, txStatus, resetTxStatus] = useAsyncProcess();
  const [txResult, setTxResult] = useState(null);
  const [baseBalance, baseBalanceStatus] = useAssetBalance(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = useAssetBalance(quoteAsset);
  const balanceStatus = useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);
  const [currentPricePerBase, currentPricePerBaseStatus] = useOnexDetailedPoolPrice({
    baseAsset,
    quoteAsset
  });
  const [currentPricePerQuote, currentPricePerQuoteStatus] = useOnexDetailedPoolPrice({
    baseAsset: quoteAsset,
    quoteAsset: baseAsset
  });
  const pair = useMemo(() => [baseAsset.id, quoteAsset.id].sort().join(","), [baseAsset.id, quoteAsset.id]);
  const [{ amountBaseInputFixed, amountQuoteInputFixed, amountLastChanged }, setInputState] = useState(DEFAULT_AMOUNT_INPUTS_STATE);
  const setAmountBaseFixed = useCallback((newAmountFixed) => {
    setInputState((state) => ({
      ...state,
      amountBaseInputFixed: newAmountFixed,
      amountLastChanged: "base"
    }));
  }, []);
  const setAmountQuoteFixed = useCallback((newAmountFixed) => {
    setInputState((state) => ({
      ...state,
      amountQuoteInputFixed: newAmountFixed,
      amountLastChanged: "quote"
    }));
  }, []);
  const setAmountBasePercentOfBalance = useCallback((percentage) => {
    const bnBalance = new BigNumber(baseBalance.amount.toString()).times(percentage / 100);
    setAmountBaseFixed(bnBalance.toFixed(0));
  }, [baseBalance.amount, setAmountBaseFixed]);
  const setAmountQuotePercentOfBalance = useCallback((percentage) => {
    const bnBalance = new BigNumber(quoteBalance.amount.toString()).times(percentage / 100);
    setAmountQuoteFixed(bnBalance.toFixed(0));
  }, [quoteBalance.amount, setAmountQuoteFixed]);
  const [{ amountB: calculatedOtherAmount, drops }, dropsStatus] = useAsyncPoll(useCallback(async () => {
    var _a;
    return (_a = onex == null ? void 0 : onex.dropCoin({
      denomA: amountLastChanged === "base" ? baseAsset.id : quoteAsset.id,
      denomB: amountLastChanged === "base" ? quoteAsset.id : baseAsset.id,
      amountA: amountLastChanged === "base" ? amountBaseInputFixed : amountQuoteInputFixed
    })) != null ? _a : {
      amountB: "0",
      drops: "0"
    };
  }, [
    onex,
    amountBaseInputFixed,
    amountQuoteInputFixed,
    amountLastChanged,
    baseAsset.id,
    quoteAsset.id
  ]), {
    amountB: "0",
    drops: "0"
  });
  const dropsToPoolRatio = useMemo(() => {
    var _a;
    const bnPoolDrops = new BigNumber((_a = pool == null ? void 0 : pool.drops) != null ? _a : 0);
    const bnDrops = new BigNumber(drops);
    const bnTotal = bnDrops.plus(bnPoolDrops);
    return bnDrops.dividedBy(bnTotal).toNumber();
  }, [pool == null ? void 0 : pool.drops, drops]);
  const amountBaseFixed = amountLastChanged === "base" ? amountBaseInputFixed : calculatedOtherAmount;
  const amountQuoteFixed = amountLastChanged === "quote" ? amountQuoteInputFixed : calculatedOtherAmount;
  const [amountQuote, amountQuoteStatus] = useDetailedBalance(quoteAsset, amountQuoteFixed);
  const [amountBase, amountBaseStatus] = useDetailedBalance(baseAsset, amountBaseFixed);
  const [amountBaseInput, setAmountBaseInput] = useFixedAmountInput({
    value: amountBaseFixed,
    onChangeValue: setAmountBaseFixed,
    meta: baseAsset.meta
  });
  const [amountQuoteInput, setAmountQuoteInput] = useFixedAmountInput({
    value: amountQuoteFixed,
    onChangeValue: setAmountQuoteFixed,
    meta: baseAsset.meta
  });
  const inputParams = useMemo(() => {
    return {
      pair,
      drops
    };
  }, [pair, drops]);
  const validationErrors = useMemo(() => {
    const errors = {};
    const bnDrops = new BigNumber(drops);
    const bnAmountBase = new BigNumber(amountBaseFixed);
    const bnAmountQuote = new BigNumber(amountQuoteFixed);
    if (bnDrops.isNaN() || bnDrops.lte(0)) {
      errors.drops = "Drops must be a number > 0";
    }
    if (bnAmountBase.isNaN() || bnAmountBase.lte(0)) {
      errors.amountBaseInput = "Must be a number > 0";
    }
    if (bnAmountQuote.isNaN() || bnAmountQuote.lte(0)) {
      errors.amountQuoteInput = "Must be a number > 0";
    }
    if (bnAmountBase.gt(baseBalance.amount.toString())) {
      errors.amountBaseInput = "Exceeds available balance";
    }
    if (bnAmountQuote.gt(quoteBalance.amount.toString())) {
      errors.amountQuoteInput = "Exceeds available balance";
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [drops, amountBaseFixed, amountQuoteFixed, baseBalance.amount, quoteBalance.amount]);
  const loadingStatus = useAsyncStatuses([balanceStatus]);
  const canSubmit = !validationErrors && !loadingStatus.pending && dropsStatus.finished;
  const submitTx = useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    if (!canSubmit)
      return Promise.resolve();
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a;
      return setTxResult((_a = await (onex == null ? void 0 : onex.createDrop(inputParams))) != null ? _a : null);
    });
  }, [onex, inputParams, canSubmit, resetTxStatus, attemptTx]);
  const reset = useCallback(() => {
    resetTxStatus();
    setTxResult(null);
  }, [resetTxStatus]);
  useEffect(reset, [baseAsset.id, quoteAsset.id, reset]);
  return {
    txStatus,
    txResult,
    loadingStatus,
    baseBalanceStatus,
    quoteBalanceStatus,
    balanceStatus,
    currentPricePerBaseStatus,
    currentPricePerQuoteStatus,
    amountBaseStatus,
    amountQuoteStatus,
    dropsStatus,
    baseAsset,
    quoteAsset,
    pool,
    drops,
    amountBaseInput,
    amountQuoteInput,
    currentPricePerBase,
    currentPricePerQuote,
    amountBase,
    amountQuote,
    baseBalance,
    quoteBalance,
    dropsToPoolRatio,
    validationErrors,
    canSubmit,
    setAmountBaseInput,
    setAmountQuoteInput,
    setAmountBasePercentOfBalance,
    setAmountQuotePercentOfBalance,
    submitTx,
    reset
  };
}

function useLiquidityPairAddDropsState(props) {
  const form = useAddDropsForm(props);
  return form;
}
const LiquidityPairAddDropsContext = createContext(null);
function useLiquidityPairAddDrops() {
  const state = useContext(LiquidityPairAddDropsContext);
  if (!state)
    throw new Error("LiquidityPairAddDropsContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN$3() {
  return /* @__PURE__ */ React.createElement(PlaceholderLoadingIndicator, {
    text: "Loading add drops form"
  });
}
function DEFAULT_ERROR_RENDER_FN$3(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function LiquidityPairAddDropsProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN$3,
  renderError = DEFAULT_ERROR_RENDER_FN$3,
  ...props
}) {
  const state = useLiquidityPairAddDropsState(props);
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(LiquidityPairAddDropsContext.Provider, {
    value: state
  }, children);
}

function useCreatePoolForm({
  baseAsset,
  quoteAsset
}) {
  const onex = useOnexClient();
  const [attemptTx, txStatus, resetTxStatus] = useAsyncProcess();
  const [txResult, setTxResult] = useState(null);
  const [baseBalance, baseBalanceStatus] = useAssetBalance(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = useAssetBalance(quoteAsset);
  const balanceStatus = useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);
  const {
    price,
    inversePrice,
    amountBaseFixed,
    amountQuoteFixed,
    pricePerBase,
    pricePerBaseStatus,
    inversePricePerBase,
    inversePricePerBaseStatus,
    setPrice,
    setAmountBaseFixed,
    setAmountQuoteFixed,
    reset: resetPriceInputs
  } = usePriceInputs({ baseAsset, quoteAsset, updateOtherAsset: false });
  const setAmountBasePercentOfBalance = useCallback((percentage) => {
    const bnBalance = new BigNumber(baseBalance.amount.toString()).times(percentage / 100);
    setAmountBaseFixed(bnBalance.toFixed(0));
  }, [baseBalance.amount, setAmountBaseFixed]);
  const setAmountQuotePercentOfBalance = useCallback((percentage) => {
    const bnBalance = new BigNumber(quoteBalance.amount.toString()).times(percentage / 100);
    setAmountQuoteFixed(bnBalance.toFixed(0));
  }, [quoteBalance.amount, setAmountQuoteFixed]);
  const [amountBaseInput, setAmountBaseInput] = useFixedAmountInput({
    value: amountBaseFixed,
    onChangeValue: setAmountBaseFixed,
    meta: baseAsset.meta
  });
  const [amountQuoteInput, setAmountQuoteInput] = useFixedAmountInput({
    value: amountQuoteFixed,
    onChangeValue: setAmountQuoteFixed,
    meta: baseAsset.meta
  });
  const [amountQuote, amountQuoteStatus] = useDetailedBalance(quoteAsset, amountQuoteFixed);
  const [amountBase, amountBaseStatus] = useDetailedBalance(baseAsset, amountBaseFixed);
  const loadingStatus = useAsyncStatuses([balanceStatus]);
  const inputParams = useMemo(() => {
    return {
      coinA: baseAsset.id,
      coinB: quoteAsset.id,
      coinAmountA: amountBaseFixed,
      coinAmountB: amountQuoteFixed
    };
  }, [baseAsset.id, quoteAsset.id, amountBaseFixed, amountQuoteFixed]);
  const validationErrors = useMemo(() => {
    const errors = {};
    const bnPrice = new BigNumber(price);
    const bnAmountA = new BigNumber(inputParams.coinAmountA);
    const bnAmountB = new BigNumber(inputParams.coinAmountB);
    if (!inputParams.coinA)
      errors.baseAsset = "Base asset must be set";
    if (!inputParams.coinB)
      errors.quoteAsset = "Quote asset must be set";
    if (bnPrice.isNaN() || bnPrice.lte(0)) {
      errors.price = "Must be a number > 0";
    }
    if (bnAmountA.isNaN() || bnAmountA.lte(0)) {
      errors.amountBaseInput = "Amount must be a number > 0";
    }
    if (bnAmountB.isNaN() || bnAmountB.lte(0)) {
      errors.amountQuoteInput = "Amount must be a number > 0";
    }
    if (bnAmountA.gt(baseBalance.amount.toString())) {
      errors.amountBaseInput = "Exceeds available balance";
    }
    if (bnAmountB.gt(quoteBalance.amount.toString())) {
      errors.amountQuoteInput = "Exceeds available balance";
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [price, inputParams, baseBalance.amount, quoteBalance.amount]);
  const canSubmit = !validationErrors && !loadingStatus.pending;
  const submitTx = useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    if (!canSubmit)
      return Promise.resolve();
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a;
      return setTxResult((_a = await (onex == null ? void 0 : onex.createPool(inputParams))) != null ? _a : null);
    });
  }, [onex, inputParams, canSubmit, resetTxStatus, attemptTx]);
  const reset = useCallback(() => {
    resetTxStatus();
    setTxResult(null);
    resetPriceInputs();
  }, [resetPriceInputs, resetTxStatus]);
  useEffect(reset, [baseAsset.id, quoteAsset.id, reset]);
  return {
    txStatus,
    txResult,
    loadingStatus,
    balanceStatus,
    pricePerBaseStatus,
    inversePricePerBaseStatus,
    amountQuoteStatus,
    amountBaseStatus,
    baseAsset,
    quoteAsset,
    price,
    inversePrice,
    amountBaseInput,
    amountQuoteInput,
    validationErrors,
    canSubmit,
    baseBalance,
    quoteBalance,
    pricePerBase,
    pricePerQuote: inversePricePerBase,
    amountQuote,
    amountBase,
    setPrice,
    setAmountBaseInput,
    setAmountQuoteInput,
    setAmountBasePercentOfBalance,
    setAmountQuotePercentOfBalance,
    submitTx,
    reset
  };
}

function useLiquidityPairCreatePoolState(props) {
  const form = useCreatePoolForm(props);
  return form;
}
const LiquidityPairCreatePoolContext = createContext(null);
function useLiquidityPairCreatePool() {
  const state = useContext(LiquidityPairCreatePoolContext);
  if (!state)
    throw new Error("LiquidityPairCreatePoolContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN$2() {
  return /* @__PURE__ */ React.createElement(PlaceholderLoadingIndicator, {
    text: "Loading add drops form"
  });
}
function DEFAULT_ERROR_RENDER_FN$2(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function LiquidityPairCreatePoolProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN$2,
  renderError = DEFAULT_ERROR_RENDER_FN$2,
  ...props
}) {
  const state = useLiquidityPairCreatePoolState(props);
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(LiquidityPairCreatePoolContext.Provider, {
    value: state
  }, children);
}

function useRedeemDropForm({ dropId = "" } = {}) {
  const onex = useOnexClient();
  const [attemptTx, txStatus, resetTxStatus] = useAsyncProcess();
  const [txResult, setTxResult] = useState(null);
  const validationErrors = useMemo(() => {
    const errors = {};
    const bnDropId = new BigNumber(dropId);
    if (bnDropId.isNaN() || bnDropId.lte(0)) {
      errors.dropId = "Coin A amount must be a number > 0";
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [dropId]);
  const canSubmit = !validationErrors;
  const doRedeem = useCallback((id) => {
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a;
      return setTxResult((_a = await (onex == null ? void 0 : onex.redeemDrop({ uid: id != null ? id : dropId }))) != null ? _a : null);
    });
  }, [onex, resetTxStatus, attemptTx, dropId]);
  const submitTx = useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    if (!canSubmit)
      return Promise.resolve();
    return doRedeem(dropId);
  }, [canSubmit, dropId, doRedeem]);
  const reset = useCallback(() => {
    resetTxStatus();
    setTxResult(null);
  }, [resetTxStatus]);
  useEffect(reset, [dropId, reset]);
  return {
    txStatus,
    txResult,
    canSubmit,
    validationErrors,
    dropId,
    doRedeem,
    submitTx,
    reset
  };
}

function useOnexLiquidity({
  baseAsset,
  quoteAsset
}) {
  const redeem = useRedeemDropForm();
  const unknownBaseBalance = useMemo(() => ({ ...OnomyHubConstants.UNKNOWN_BALANCE, asset: baseAsset }), [baseAsset.id, baseAsset.chain.name]);
  const unknownQuoteBalance = useMemo(() => ({ ...OnomyHubConstants.UNKNOWN_BALANCE, asset: quoteAsset }), [quoteAsset.id, baseAsset.chain.name]);
  const [pool, poolStatus] = useOnexPoolDetail({ baseAsset, quoteAsset });
  const [myDrops, myDropsStatus] = useOnexMyDropsForPool(pool);
  const basePrincipalAmounts = useMemo(() => myDrops.map((drop) => drop.principalBase), [myDrops]);
  const quotePrincipalAmounts = useMemo(() => myDrops.map((drop) => drop.principalQuote), [myDrops]);
  const baseRewardAmounts = useMemo(() => myDrops.map((drop) => drop.rewardBase), [myDrops]);
  const quoteRewardAmounts = useMemo(() => myDrops.map((drop) => drop.rewardQuote), [myDrops]);
  const [[basePrincipal = unknownBaseBalance], basePrincipalStatus] = useBalanceTotals(basePrincipalAmounts);
  const [[quotePrincipal = unknownQuoteBalance], quotePrincipalStatus] = useBalanceTotals(quotePrincipalAmounts);
  const [[baseReward = unknownBaseBalance], baseRewardStatus] = useBalanceTotals(baseRewardAmounts);
  const [[quoteReward = unknownQuoteBalance], quoteRewardStatus] = useBalanceTotals(quoteRewardAmounts);
  const [[baseTotal = unknownBaseBalance], baseTotalStatus] = useBalanceTotals([
    basePrincipal,
    baseReward
  ]);
  const [[quoteTotal = unknownQuoteBalance], quoteTotalStatus] = useBalanceTotals([
    quotePrincipal,
    quoteReward
  ]);
  const fiatTotal = baseTotal.fiatValue + quoteTotal.fiatValue;
  const totalsStatus = useAsyncStatuses([
    basePrincipalStatus,
    quotePrincipalStatus,
    baseRewardStatus,
    quoteRewardStatus,
    baseTotalStatus,
    quoteTotalStatus
  ]);
  const totalRewardRatio = useMemo(() => {
    const bnPrinciple = new BigNumber(basePrincipal.amount.toString());
    const bnReward = new BigNumber(baseReward.amount.toString());
    const bnTotal = bnReward.plus(bnPrinciple);
    return bnReward.dividedBy(bnTotal).toNumber();
  }, [basePrincipal.amount, baseReward.amount]);
  const loadingStatus = useAsyncStatuses([poolStatus, myDropsStatus, totalsStatus]);
  return {
    loadingStatus,
    poolStatus,
    myDropsStatus,
    basePrincipalStatus,
    quotePrincipalStatus,
    baseRewardStatus,
    quoteRewardStatus,
    totalsStatus,
    pool,
    myDrops,
    redeem,
    basePrincipal,
    quotePrincipal,
    baseReward,
    quoteReward,
    baseTotal,
    quoteTotal,
    fiatTotal,
    totalRewardRatio
  };
}

function useLiquidityPairState({
  quoteAsset,
  baseAsset
}) {
  const liquidity = useOnexLiquidity({ baseAsset, quoteAsset });
  return {
    ...liquidity,
    baseAsset,
    quoteAsset
  };
}
const LiquidityPairContext = createContext(null);
function useLiquidityPair() {
  const state = useContext(LiquidityPairContext);
  if (!state)
    throw new Error("LiquidityPairContext missing");
  return state;
}
const DEFAULT_LOADING_RENDER_FN$1 = () => /* @__PURE__ */ React.createElement(PlaceholderLoadingIndicator, {
  text: "Loading liquidity pair"
});
const DEFAULT_ERROR_RENDER_FN$1 = (error) => /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
  error
});
function LiquidityPairProvider({
  quoteAsset,
  baseAsset,
  renderLoading = DEFAULT_LOADING_RENDER_FN$1,
  renderError = DEFAULT_ERROR_RENDER_FN$1,
  children
}) {
  const state = useLiquidityPairState({ baseAsset, quoteAsset });
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(LiquidityPairContext.Provider, {
    value: state
  }, children);
}

function getAssetKey(asset) {
  return [asset.chain.name, asset.id].join("|");
}
function useWalletBridgeFormState() {
  const { hub } = useOnomyHub();
  const { supportedAssets } = useOnomyApp();
  if (!hub)
    throw new Error("onomy hub not present");
  const bridgeAssetMap = useMemo(() => {
    function findAsset(asset) {
      const id = serializeAsset(asset);
      return supportedAssets.find((a) => serializeAsset(a) === id);
    }
    return hub.bridges.reduce((res, bridge) => {
      const bridgeFromAsset = findAsset(bridge.getFromAsset());
      const bridgeToAsset = findAsset(bridge.getToAsset());
      if (!bridgeFromAsset || !bridgeToAsset)
        return res;
      const fromAssetKey = getAssetKey(bridgeFromAsset);
      const toAssetKey = getAssetKey(bridgeToAsset);
      const fromAssetRecord = res[fromAssetKey] = res[fromAssetKey] || {
        fromAsset: bridgeFromAsset,
        toAssets: {}
      };
      fromAssetRecord.toAssets[toAssetKey] = bridgeToAsset;
      return res;
    }, {});
  }, [hub.bridges, supportedAssets]);
  const fromAssets = useMemo(() => Object.values(bridgeAssetMap).map((v) => v.fromAsset), [bridgeAssetMap]);
  const [fromAsset, setFromAsset] = useState(OnomyHubConstants.NOM_ASSET);
  const toAssets = useMemo(() => {
    var _a, _b;
    const key = getAssetKey(fromAsset);
    return Object.values((_b = (_a = bridgeAssetMap[key]) == null ? void 0 : _a.toAssets) != null ? _b : {});
  }, [fromAsset, bridgeAssetMap]);
  const [toAsset, setToAsset] = useState(OnomyHubConstants.ONEX_NOM_ASSET);
  useEffect(() => {
    var _a, _b;
    const cur = serializeAsset(toAsset);
    setToAsset((_b = (_a = toAssets.find((a) => serializeAsset(a) === cur)) != null ? _a : toAssets[0]) != null ? _b : toAsset);
  }, [toAssets]);
  const form = useBridgeForm({
    fromAsset,
    toAsset
  });
  const [amountDisplayValue, setAmountDisplayValue] = useFixedAmountInput({
    value: form.bridgeAmountFixed,
    onChangeValue: form.setBridgeAmountFixed,
    meta: form.fromAsset.meta
  });
  return {
    ...form,
    fromAssets,
    toAssets,
    amountDisplayValue,
    setAmountDisplayValue,
    setFromAsset,
    setToAsset
  };
}
const WalletBridgeFormContext = createContext(null);
function useWalletBridgeForm() {
  const state = useContext(WalletBridgeFormContext);
  if (!state)
    throw new Error("ValidatorDetailContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN() {
  return /* @__PURE__ */ React.createElement(PlaceholderLoadingIndicator, {
    text: "Loading bridge form"
  });
}
function DEFAULT_ERROR_RENDER_FN(error) {
  return /* @__PURE__ */ React.createElement(PlaceholderErrorDisplay, {
    error
  });
}
function WalletBridgeFormProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN
}) {
  const state = useWalletBridgeFormState();
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React.createElement(WalletBridgeFormContext.Provider, {
    value: state
  }, children);
}

export { ClaimRewardProvider, DelegateAssetProvider, ExchangeCancelOrderProvider, ExchangePairProvider, ExchangePairProviderInner, GovernanceProvider, LiquidityPairAddDropsProvider, LiquidityPairCreatePoolProvider, LiquidityPairProvider, MnemonicWalletContext, MnemonicWalletProvider, MoveDelegationContext, MoveDelegationProvider, OnomyCoreProvider, OnomyDashboardProvider, ProposalCreateProvider, ProposalDepositContext, ProposalDepositProvider, ProposalDetailProvider, ProposalVoteContext, ProposalVoteProvider, TransferAssetProvider, ValidatorDetailProvider, WalletBridgeFormProvider, governanceUtils, setOnomyEcosystemConfig, useAsset, useAssetDelegations, useAvailableBalance, useChainDelegations, useClaimReward, useDelegateAsset, useDenomMetadataProposalContent, useExchangeCancelOrder, useExchangePair, useFundAccountProposalContent, useGovernance, useIndexedOrderBookForm, useLiquidityPair, useLiquidityPairAddDrops, useLiquidityPairCreatePool, useMnemonicWallet, useMoveDelegation, useOnexDayTradeFrame, useOnexTradingViewData, useOnomyApp, useOnomyDashboard, useProposalCreate, useProposalDeposit, useProposalDetail, useProposalVote, useRewardBalance, useStakedBalance, useTransferAsset, useUnknownProposalContent, useUnstakingBalance, useValidatorDetail, useWalletBridgeForm };
