import React, { createContext, useContext } from 'react';
import { useCreatePoolForm } from '../forms/useCreatePoolForm';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';

function useLiquidityPairCreatePoolState(props: Parameters<typeof useCreatePoolForm>[0]) {
  const form = useCreatePoolForm(props);
  return form;
}

export type LiquidityPairCreatePoolContextType = ReturnType<typeof useLiquidityPairCreatePoolState>;

const LiquidityPairCreatePoolContext = createContext<LiquidityPairCreatePoolContextType>(
  null as any
);

export function useLiquidityPairCreatePool() {
  const state = useContext(LiquidityPairCreatePoolContext);
  if (!state) throw new Error('LiquidityPairCreatePoolContext missing');
  return state;
}

// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_LOADING_RENDER_FN() {
  return <PlaceholderLoadingIndicator text="Loading add drops form" />;
}
// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_ERROR_RENDER_FN(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function LiquidityPairCreatePoolProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
  ...props
}: {
  children: JSX.Element | JSX.Element[];
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
} & Parameters<typeof useLiquidityPairCreatePoolState>[0]) {
  const state = useLiquidityPairCreatePoolState(props);

  if (state.loadingStatus.pending) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);
  return (
    <LiquidityPairCreatePoolContext.Provider value={state}>
      {children}
    </LiquidityPairCreatePoolContext.Provider>
  );
}
