import { CosmosProposalType, OnomyProposals } from '@onomy/client';
import { Blockchain } from '@onomy/frontend-relay-types';
import { useCallback, useMemo, useState } from 'react';

const BLANK_JSON = '{}';

export function useUnknownProposalContent({
  chain,
  type,
}: {
  chain: Blockchain;
  type: CosmosProposalType;
}) {
  const [json, setJson] = useState(BLANK_JSON);

  const parsed = useMemo(() => {
    try {
      return JSON.parse(json);
    } catch (e) {
      return null;
    }
  }, [json]);

  const encoded = useMemo(() => {
    if (!parsed) return null;
    try {
      return (type.encode as any)(parsed);
    } catch (e) {
      return null;
    }
  }, [parsed, type]);

  const formErrors = useMemo(() => {
    const errors: { json?: string } = {};

    if (!encoded) errors.json = 'Invalid proposal content';
    if (!parsed) errors.json = 'Invalid JSON';

    return Object.keys(errors).length > 0 ? errors : null;
  }, [parsed, encoded]);

  const canSubmit = !formErrors;

  const reset = useCallback(() => {
    setJson(BLANK_JSON);
  }, []);

  return {
    json,
    parsed,

    chain,
    type,

    setJson,

    encoded,

    formErrors,
    canSubmit,

    reset,
  };
}
